# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license*.

import datetime
import email
import io
import logging
import re
from email.header import decode_header

import eve
from flask import current_app as app, json
from pytz import timezone

import superdesk
from superdesk import get_resource_service
from superdesk.errors import IngestEmailError
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import EmailFeedParser
from superdesk.io.iptc import subject_codes
from superdesk.media.media_operations import process_file_from_stream
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG, SIGN_OFF, BYLINE, FORMATS, FORMAT
from superdesk.metadata.utils import generate_guid
from superdesk.users.errors import UserNotRegisteredException
from superdesk.utc import utcnow, get_date
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.filemeta import set_filemeta
from superdesk.text_utils import sanitize_html


logger = logging.getLogger(__name__)


email_regex = re.compile("^.*<(.*)>$")


class EMailRFC822FeedParser(EmailFeedParser):
    """
    Feed Parser which can parse if the feed is in RFC 822 format.
    """

    NAME = "email_rfc822"

    label = "EMail RFC822 Parser"

    def __init__(self):
        self.parser_app = app

    def can_parse(self, email_message):
        for response_part in email_message:
            if isinstance(response_part, tuple):
                msg = email.message_from_bytes(response_part[1])
                return self.parse_header(msg["from"]) != ""

        return False

    def parse(self, data, provider=None):
        config = provider.get("config", {})
        # If the channel is configured to process structured email generated from a google form
        if config.get("formatted", False):
            return self._parse_formatted_email(data, provider)
        try:
            new_items = []
            # create an item for the body text of the email
            # either text or html
            item = dict()
            item[ITEM_TYPE] = CONTENT_TYPE.TEXT
            item["versioncreated"] = utcnow()

            comp_item = None

            # a list to keep the references to the attachments
            refs = []

            html_body = None
            text_body = None

            for response_part in data:
                if isinstance(response_part, tuple):
                    msg = email.message_from_bytes(response_part[1])
                    item["headline"] = self.parse_header(msg["subject"])
                    field_from = self.parse_header(msg["from"])
                    item["original_source"] = field_from
                    try:
                        if email_regex.findall(field_from):
                            email_address = email_regex.findall(field_from)[0]
                            user = get_resource_service("users").get_user_by_email(email_address)
                            item["original_creator"] = user[eve.utils.config.ID_FIELD]
                    except UserNotRegisteredException:
                        pass
                    item["guid"] = msg["Message-ID"]
                    date_tuple = email.utils.parsedate_tz(msg["Date"])
                    if date_tuple:
                        dt = datetime.datetime.utcfromtimestamp(email.utils.mktime_tz(date_tuple))
                        dt = dt.replace(tzinfo=timezone("utc"))
                        item["firstcreated"] = dt

                    # this will loop through all the available multiparts in mail
                    for part in msg.walk():
                        if part.get_content_type() == "text/plain":
                            body = part.get_payload(decode=True)
                            try:
                                # if we don't know the charset just have a go!
                                if part.get_content_charset() is None:
                                    text_body = body.decode()
                                else:
                                    charset = part.get_content_charset()
                                    text_body = body.decode(charset)
                                continue
                            except Exception as ex:
                                logger.exception(
                                    "Exception parsing text body for {0} from {1}: {2}".format(
                                        item["headline"], field_from, ex
                                    )
                                )
                                continue
                        if part.get_content_type() == "text/html":
                            body = part.get_payload(decode=True)
                            try:
                                if part.get_content_charset() is None:
                                    html_body = body.decode()
                                else:
                                    charset = part.get_content_charset()
                                    html_body = body.decode(charset)
                                html_body = sanitize_html(html_body)
                                continue
                            except Exception as ex:
                                logger.exception(
                                    "Exception parsing html body for {0} from {1}: {2}".format(
                                        item["headline"], field_from, ex
                                    )
                                )
                                continue
                        if part.get_content_maintype() == "multipart":
                            continue
                        if part.get("Content-Disposition") is None:
                            continue
                        # we are only going to pull off image attachments at this stage
                        if part.get_content_maintype() != "image":
                            continue

                        fileName = part.get_filename()
                        if bool(fileName):
                            image = part.get_payload(decode=True)
                            content = io.BytesIO(image)
                            res = process_file_from_stream(content, part.get_content_type())
                            file_name, content_type, metadata = res
                            if content_type == "image/gif" or content_type == "image/png":
                                continue
                            content.seek(0)
                            image_id = self.parser_app.media.put(
                                content, filename=fileName, content_type=content_type, metadata=metadata
                            )
                            renditions = {"baseImage": {"href": image_id}}

                            # if we have not got a composite item then create one
                            if not comp_item:
                                comp_item = dict()
                                comp_item[ITEM_TYPE] = CONTENT_TYPE.COMPOSITE
                                comp_item["guid"] = generate_guid(type=GUID_TAG)
                                comp_item["versioncreated"] = utcnow()
                                comp_item["groups"] = []
                                comp_item["headline"] = item["headline"]
                                comp_item["groups"] = []
                                comp_item["original_source"] = item["original_source"]
                                if "original_creator" in item:
                                    comp_item["original_creator"] = item["original_creator"]

                                # create a reference to the item that stores the body of the email
                                item_ref = {
                                    "guid": item["guid"],
                                    "residRef": item["guid"],
                                    "headline": item["headline"],
                                    "location": "ingest",
                                    "itemClass": "icls:text",
                                    "original_source": item["original_source"],
                                }
                                if "original_creator" in item:
                                    item_ref["original_creator"] = item["original_creator"]
                                refs.append(item_ref)

                            media_item = dict()
                            media_item["guid"] = generate_guid(type=GUID_TAG)
                            media_item["versioncreated"] = utcnow()
                            media_item[ITEM_TYPE] = CONTENT_TYPE.PICTURE
                            media_item["renditions"] = renditions
                            media_item["mimetype"] = content_type
                            set_filemeta(media_item, metadata)
                            media_item["slugline"] = fileName
                            if text_body is not None:
                                media_item["body_html"] = text_body
                            media_item["headline"] = item["headline"]
                            media_item["original_source"] = item["original_source"]
                            if "original_creator" in item:
                                media_item["original_creator"] = item["original_creator"]
                            new_items.append(media_item)

                            # add a reference to this item in the composite item
                            media_ref = {
                                "guid": media_item["guid"],
                                "residRef": media_item["guid"],
                                "headline": fileName,
                                "location": "ingest",
                                "itemClass": "icls:picture",
                                "original_source": item["original_source"],
                            }
                            if "original_creator" in item:
                                media_ref["original_creator"] = item["original_creator"]
                            refs.append(media_ref)

            if html_body:
                item["body_html"] = html_body
            else:
                item["body_html"] = "<pre>" + text_body + "</pre>"
                item[FORMAT] = FORMATS.PRESERVED

            # if there is composite item then add the main group and references
            if comp_item:
                grefs = {"refs": [{"idRef": "main"}], "id": "root", "role": "grpRole:NEP"}
                comp_item["groups"].append(grefs)

                grefs = {"refs": refs, "id": "main", "role": "grpRole:Main"}
                comp_item["groups"].append(grefs)

                new_items.append(comp_item)

            new_items.append(item)
            return new_items
        except Exception as ex:
            raise IngestEmailError.emailParseError(ex, provider)

    def parse_header(self, field):
        try:
            hdr = decode_header(field)
            encoding = hdr[0][1]
            if encoding and hdr:
                parsed_field = hdr[0][0].decode(encoding)
            else:
                parsed_field = hdr[0][0]
        except Exception:
            try:
                parsed_field = str(field)
            except Exception:
                parsed_field = "Unknown"
            pass
        return parsed_field

    def _attr_name_whitelisted(self, attr_name):
        return attr_name.lower() in ["href", "style", "color", "size", "bgcolor", "border"]

    def safe_css(self, attr, css):
        if attr == "style":
            return re.sub("(width|height):[^;]+;", "", css)
        return css

    def _expand_category(self, item, mail_item):
        """Given a list of category names in the incoming email try to look them up to match category codes.

        If there is a subject associated with the category it will insert that into the item as well

        :param item:
        :param mail_item:
        :return: An item populated with category codes
        """
        anpa_categories = superdesk.get_resource_service("vocabularies").find_one(req=None, _id="categories")
        if anpa_categories:
            for mail_category in mail_item.get("Category").split(","):
                for anpa_category in anpa_categories["items"]:
                    if (
                        anpa_category["is_active"] is True
                        and mail_category.strip().lower() == anpa_category["name"].lower()
                    ):
                        if "anpa_category" not in item:
                            item["anpa_category"] = list()
                        item["anpa_category"].append({"qcode": anpa_category["qcode"]})
                        if anpa_category.get("subject"):
                            if "subject" not in item:
                                item["subject"] = list()
                            item["subject"].append(
                                {
                                    "qcode": anpa_category.get("subject"),
                                    "name": subject_codes[anpa_category.get("subject")],
                                }
                            )
                        break

    def _parse_formatted_email(self, data, provider):
        """Construct an item from an email that was constructed as a notification from a google form submission.

        The google form submits to a google sheet, this sheet creates the email as a notification

        :param data:
        :param provider:
        :return: A list of 1 item
        """
        try:
            item = dict()
            item[ITEM_TYPE] = CONTENT_TYPE.TEXT
            item["versioncreated"] = utcnow()
            for response_part in data:
                if isinstance(response_part, tuple):
                    msg = email.message_from_bytes(response_part[1])
                    # Check that the subject line matches what we expect, ignore it if not
                    if self.parse_header(msg["subject"]) != "Formatted Editorial Story":
                        return []

                    item["guid"] = msg["Message-ID"]
                    date_tuple = email.utils.parsedate_tz(msg["Date"])
                    if date_tuple:
                        dt = datetime.datetime.utcfromtimestamp(email.utils.mktime_tz(date_tuple))
                        dt = dt.replace(tzinfo=timezone("utc"))
                        item["firstcreated"] = dt

                    for part in msg.walk():
                        if part.get_content_type() == "text/plain":
                            body = part.get_payload(decode=True)
                            # if we don't know the charset just have a go!
                            if part.get_content_charset() is None:
                                json_str = body.decode().replace("\r\n", "").replace("  ", " ")
                            else:
                                charset = part.get_content_charset()
                                json_str = body.decode(charset).replace("\r\n", "").replace("  ", " ")

                            mail_item = dict((k, v[0]) for k, v in json.loads(json_str).items())

                            self._expand_category(item, mail_item)

                            item["original_source"] = mail_item.get("Username", mail_item.get("Email Address", ""))
                            item["headline"] = mail_item.get("Headline", "")
                            item["abstract"] = mail_item.get("Abstract", "")
                            item["slugline"] = mail_item.get("Slugline", "")
                            item["body_html"] = "<p>" + mail_item.get("Body", "").replace("\n", "</p><p>") + "</p>"

                            default_source = app.config.get("DEFAULT_SOURCE_VALUE_FOR_MANUAL_ARTICLES")
                            city = mail_item.get("Dateline", "")
                            cities = app.locators.find_cities()
                            located = [c for c in cities if c["city"].lower() == city.lower()]
                            item.setdefault("dateline", {})
                            item["dateline"]["located"] = (
                                located[0]
                                if len(located) > 0
                                else {"city_code": city, "city": city, "tz": "UTC", "dateline": "city"}
                            )
                            item["dateline"]["source"] = default_source
                            item["dateline"]["text"] = format_dateline_to_locmmmddsrc(
                                item["dateline"]["located"], get_date(item["firstcreated"]), source=default_source
                            )

                            if mail_item.get("Priority") != "":
                                if mail_item.get("Priority", "3").isdigit():
                                    item["priority"] = int(mail_item.get("Priority", "3"))
                                else:
                                    priority_map = superdesk.get_resource_service("vocabularies").find_one(
                                        req=None, _id="priority"
                                    )
                                    priorities = [
                                        x
                                        for x in priority_map.get("items", [])
                                        if x["name"].upper() == mail_item.get("Priority", "").upper()
                                    ]
                                    if priorities is not None and len(priorities) > 0:
                                        item["priority"] = int(priorities[0].get("qcode", "3"))
                                    else:
                                        item["priority"] = 3
                            if mail_item.get("News Value") != "":
                                item["urgency"] = int(mail_item.get("News Value", "3"))

                            # We expect the username passed corresponds to a superdesk user
                            query = {
                                "email": re.compile(
                                    "^{}$".format(mail_item.get("Username", mail_item.get("Email Address", ""))),
                                    re.IGNORECASE,
                                )
                            }
                            user = superdesk.get_resource_service("users").find_one(req=None, **query)
                            if not user:
                                logger.error(
                                    "Failed to find user for email {}".format(
                                        mail_item.get("Username", mail_item.get("Email Address", ""))
                                    )
                                )
                                raise UserNotRegisteredException()
                            item["original_creator"] = user.get("_id")
                            if BYLINE in user and user.get(BYLINE, ""):
                                item["byline"] = user.get(BYLINE)
                            item[SIGN_OFF] = user.get(SIGN_OFF)

                            # attempt to match the given desk name against the defined desks
                            query = {"name": re.compile("^{}$".format(mail_item.get("Desk", "")), re.IGNORECASE)}
                            desk = superdesk.get_resource_service("desks").find_one(req=None, **query)
                            if desk:
                                item["task"] = {"desk": desk.get("_id"), "stage": desk.get("incoming_stage")}

                            if "Place" in mail_item:
                                locator_map = superdesk.get_resource_service("vocabularies").find_one(
                                    req=None, _id="locators"
                                )
                                place = [
                                    x
                                    for x in locator_map.get("items", [])
                                    if x["qcode"] == mail_item.get("Place", "").upper()
                                ]
                                if place is not None:
                                    item["place"] = place

                            if mail_item.get("Legal flag", "") == "LEGAL":
                                item["flags"] = {"marked_for_legal": True}

                            break

            return [item]
        except Exception as ex:
            raise IngestEmailError.emailParseError(ex, provider)


register_feed_parser(EMailRFC822FeedParser.NAME, EMailRFC822FeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import pytz
from datetime import datetime, timedelta

from apps.rules.routing_rules import Weekdays
from apps.templates.content_templates import get_next_run, get_item_from_template, render_content_template
from superdesk.metadata.item import ITEM_STATE, CONTENT_STATE
from superdesk.tests import TestCase
from superdesk.utc import utcnow


class TemplatesTestCase(TestCase):
    def setUp(self):
        # now is today at 09:05:03
        self.now = datetime.utcnow().replace(hour=9, minute=5, second=3)
        self.weekdays = [day.name for day in Weekdays]

    def get_delta(self, create_at, weekdays, time_zone=None, now=None, cron_list=None):
        schedule = {"day_of_week": weekdays, "create_at": create_at, "is_active": True, "time_zone": time_zone or "UTC"}
        if cron_list:
            schedule["cron_list"] = cron_list
            schedule.pop("create_at", None)

        next_run = get_next_run(schedule, now or self.now)
        return next_run - (now or self.now).replace(second=0)

    def test_inactive_schedule(self):
        self.assertEqual(
            None, get_next_run({"is_active": False, "day_of_week": self.weekdays, "create_at": "09:15:00"})
        )

    def test_next_run_same_day_later(self):
        delta = self.get_delta("09:08:00", self.weekdays)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 179)

    def test_next_run_same_day_later_cron_list(self):
        cron_list = ["30 07 * * *", "08 09 * * *"]
        delta = self.get_delta("09:08:00", self.weekdays, cron_list=cron_list)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 179)

    def test_next_run_next_day(self):
        delta = self.get_delta("09:03:00", self.weekdays)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 3600 * 24 - 121)

    def test_next_run_next_week(self):
        delta = self.get_delta("09:03:00", [self.now.strftime("%a").upper()])
        self.assertEqual(delta.days, 6)

    def test_next_run_now(self):
        delta = self.get_delta("09:05:00", self.weekdays)
        self.assertEqual(delta.seconds, 24 * 60 * 60 - 1)

    def test_get_item_from_template(self):
        template = {
            "_id": "foo",
            "name": "test",
            "schedule_desk": "sports",
            "schedule_stage": "schedule",
            "data": {
                "headline": "Foo",
                "dateline": {
                    "located": {"city": "Sydney", "city_code": "Sydney", "tz": "Australia/Sydney"},
                    "date": "2015-10-10T10:10:10",
                },
            },
        }
        now = utcnow()
        with self.app.app_context():
            item = get_item_from_template(template)
        self.assertNotIn("_id", item)
        self.assertEqual("foo", item.get("template"))
        self.assertEqual("Foo", item.get("headline"))
        self.assertEqual(CONTENT_STATE.SUBMITTED, item.get(ITEM_STATE))
        self.assertEqual({"desk": "sports", "stage": "schedule"}, item.get("task"))
        dateline = item.get("dateline")
        self.assertEqual("Sydney", dateline["located"]["city"])
        self.assertEqual(now, dateline.get("date"))
        self.assertIn("SYDNEY", dateline.get("text"))

    def test_next_run_for_timezone(self):
        # UTC time Zero hours
        now = datetime(2018, 6, 30, 19, 0, 0, 0, tzinfo=pytz.utc)
        current_now = now + timedelta(seconds=5)
        # schedule at 06:00 AM
        delta = self.get_delta("06:00:00", self.weekdays, time_zone="Australia/Sydney", now=current_now)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 3600)

        # 30 minutes before schedule
        current_now = now + timedelta(minutes=30)
        delta = self.get_delta("06:00:00", self.weekdays, time_zone="Australia/Sydney", now=current_now)
        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 1800)

        # hour after schedule
        current_now = now + timedelta(hours=1, seconds=5)
        delta = self.get_delta("06:00:00", self.weekdays, time_zone="Australia/Sydney", now=current_now)
        self.assertEqual(delta.days, 1)


class RenderTemplateTestCase(TestCase):
    def test_render_content_template(self):
        template = {
            "_id": "foo",
            "template_name": "test",
            "template_desks": ["sports"],
            "data": {
                "headline": "Foo Template: {{item.headline}}",
                "body_html": 'This article has slugline: {{item.slugline}} and dateline: {{item.dateline["text"]}} '
                'at {{item.versioncreated | format_datetime("Australia/Sydney", "%d %b %Y %H:%S %Z")}}',
                "urgency": 1,
                "priority": 3,
                "dateline": {},
                "anpa_take_key": "this is test",
                "place": ["Australia"],
            },
        }

        item = {
            "_id": "123",
            "headline": "Test Template",
            "slugline": "Testing",
            "body_html": "This is test story",
            "dateline": {"text": "hello world"},
            "urgency": 4,
            "priority": 6,
            "versioncreated": "2015-06-01T22:54:53+0000",
            "place": ["NSW"],
        }

        updates = render_content_template(item, template)
        self.assertEqual(updates["headline"], "Foo Template: Test Template")
        self.assertEqual(updates["urgency"], 1)
        self.assertEqual(updates["priority"], 3)
        self.assertEqual(
            updates["body_html"],
            "This article has slugline: Testing and dateline: " "hello world at 02 Jun 2015 08:53 AEST",
        )
        self.assertListEqual(updates["place"], ["Australia"])

    def test_headline_strip_tags(self):
        template = {"data": {"headline": " test\nit<br>"}}

        updates = render_content_template({}, template)
        self.assertEqual("test it", updates["headline"])

        item = get_item_from_template(template)
        self.assertEqual("test it", item["headline"])



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : administrator
# Creation: 2021-02-10 18:44

from superdesk import get_resource_service
from superdesk.commands.data_updates import BaseDataUpdate
from eve.utils import config


class DataUpdate(BaseDataUpdate):

    resource = "contacts"

    def forwards(self, mongodb_collection, mongodb_database):
        countries = get_resource_service("vocabularies").find_one(req=None, _id="countries")
        for document in mongodb_collection.find({"country": {"$exists": True}}):
            if document.get("country") and countries:
                country = [t for t in countries.get("items") if t.get("name") == document["country"]]
                if country:
                    mongodb_collection.update(
                        {"_id": document.get(config.ID_FIELD)},
                        {
                            "$set": {
                                "country": {
                                    "name": country[0].get("name"),
                                    "qcode": country[0].get("qcode"),
                                    "translations": country[0].get("translations"),
                                }
                            }
                        },
                    )

    def backwards(self, mongodb_collection, mongodb_database):
        raise NotImplementedError()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""NINJS formatter for Superdesk

.. versionadded:: 1.8
    Added *source* field.

.. versionadded:: 1.7
    Added *ednote* field.
    Added *signal* field.
    Added *genre* field.

.. versionchanged:: 1.7
    Fixed copyrightholder/copyrightnotice handling to be consistent with newsml.
    Fixed place property qcode should be code.
    Output profile name instead of _id in profile field.

.. versionadded:: 1.6
    Added *evolvedfrom* field to ninjs output.

"""


import json
import superdesk
import logging
import re
from typing import Tuple

from flask import current_app as app
from eve.utils import config
from superdesk.publish.formatters import Formatter
from superdesk.errors import FormatterError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO, GUID_FIELD, ASSOCIATIONS
from superdesk.metadata.packages import RESIDREF, GROUP_ID, GROUPS, ROOT_GROUP, REFS
from superdesk.utils import json_serialize_datetime_objectId
from superdesk.media.renditions import get_renditions_spec
from superdesk.vocabularies import is_related_content
from apps.archive.common import get_utc_schedule
from superdesk import text_utils
from collections import OrderedDict

logger = logging.getLogger(__name__)
# this regex match the way custom media fields are put in associations (i.e. how the key
# is generated). This is legacy, and can hardly be changed without risking to break
# instances in production.
MEDIA_FIELD_RE = re.compile(r"(?P<field_id>\S+)--(?P<version>\d+)$")
EXTRA_ITEMS = "extra_items"


def filter_empty_vals(data):
    """Filter out `None` values from a given dict."""
    return dict(filter(lambda x: x[1], data.items()))


def get_locale_name(item, language):
    try:
        return item["translations"]["name"][language]
    except (KeyError, TypeError):
        return item.get("name", "")


def format_cv_item(item, language):
    """Format item from controlled vocabulary for output."""
    return filter_empty_vals(
        {"code": item.get("qcode"), "name": get_locale_name(item, language), "scheme": item.get("scheme")}
    )


class NINJSFormatter(Formatter):
    """
    The schema we use for the ninjs format is an extension
    of `the standard ninjs schema <http://www.iptc.org/std/ninjs/ninjs-schema_1.1.json>`_.

    *Changes from ninjs schema*:

    * ``uri`` was replaced by ``guid``: ``uri`` should be the resource identifier on the web
        but since the item was not published yet it can't be determined at this point
    * added ``priority`` field
    * added ``service`` field
    * added ``slugline`` field
    * added ``keywords`` field
    * added ``evolvedfrom`` field
    * added ``source`` field

    Associations dictionary may contain entire items like
    in `ninjs example <http://dev.iptc.org/ninjs-Examples-3>`_ or just the item ``guid``
    and ``type``. In the latest case the items are sent separately before the package item.
    """

    direct_copy_properties: Tuple[str, ...] = (
        "versioncreated",
        "usageterms",
        "language",
        "headline",
        "copyrightnotice",
        "urgency",
        "pubstatus",
        "mimetype",
        "copyrightholder",
        "ednote",
        "body_text",
        "body_html",
        "slugline",
        "keywords",
        "firstcreated",
        "firstpublished",
        "source",
        "extra",
        "annotations",
    )

    rendition_properties = ("href", "width", "height", "mimetype", "poi", "media")
    vidible_fields = {field: field for field in rendition_properties}
    vidible_fields.update(
        {
            "url": "href",
            "duration": "duration",
            "mimeType": "mimetype",
            "size": "size",
        }
    )

    def __init__(self):
        self.format_type = "ninjs"
        self.can_preview = True
        self.can_export = True
        self.internal_renditions = ["original"]

    def format(self, article, subscriber, codes=None):
        try:
            pub_seq_num = superdesk.get_resource_service("subscribers").generate_sequence_number(subscriber)

            ninjs = self._transform_to_ninjs(article, subscriber)
            return [(pub_seq_num, json.dumps(ninjs, default=json_serialize_datetime_objectId))]
        except Exception as ex:
            raise FormatterError.ninjsFormatterError(ex, subscriber)

    def _transform_to_ninjs(self, article, subscriber, recursive=True):
        ninjs = {
            "guid": article.get(GUID_FIELD, article.get("uri")),
            "version": str(article.get(config.VERSION, 1)),
            "type": self._get_type(article),
        }

        if article.get("byline"):
            ninjs["byline"] = article["byline"]

        located = article.get("dateline", {}).get("located", {})
        if located:
            ninjs["located"] = located.get("city", "")

        for copy_property in self.direct_copy_properties:
            if article.get(copy_property) is not None:
                ninjs[copy_property] = article[copy_property]

        if "body_text" not in article and "alt_text" in article:
            ninjs["body_text"] = article["alt_text"]

        if "title" in article:
            ninjs["headline"] = article["title"]

        if article.get("body_html"):
            ninjs["body_html"] = self.append_body_footer(article)

        if article.get("description"):
            ninjs["description_html"] = self.append_body_footer(article)

        if article.get("place"):
            ninjs["place"] = self._format_place(article)

        if article.get("profile"):
            ninjs["profile"] = self._format_profile(article["profile"])

        extra_items = None
        if recursive:
            if article[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                ninjs[ASSOCIATIONS] = self._get_associations(article, subscriber)
                if article.get(ASSOCIATIONS):
                    associations, extra_items = self._format_related(article, subscriber)
                    ninjs[ASSOCIATIONS].update(associations)
            elif article.get(ASSOCIATIONS):
                ninjs[ASSOCIATIONS], extra_items = self._format_related(article, subscriber)
        elif article.get(ASSOCIATIONS) and recursive:
            ninjs[ASSOCIATIONS], extra_items = self._format_related(article, subscriber)
        if extra_items:
            ninjs.setdefault(EXTRA_ITEMS, {}).update(extra_items)

        if article.get("embargoed"):
            ninjs["embargoed"] = article["embargoed"].isoformat()

        if article.get(EMBARGO):  # embargo set in superdesk overrides ingested one
            ninjs["embargoed"] = get_utc_schedule(article, EMBARGO).isoformat()

        if article.get("priority"):
            ninjs["priority"] = article["priority"]
        else:
            ninjs["priority"] = 5

        if article.get("subject"):
            ninjs["subject"] = self._get_subject(article)

        if article.get("anpa_category"):
            ninjs["service"] = self._get_service(article)
        if article.get("renditions"):
            ninjs["renditions"] = self._get_renditions(article)
        elif "url" in article:
            ninjs["renditions"] = self._generate_renditions(article)

        if "order" in article:
            ninjs["order"] = article["order"]

        # SDPA-317
        if "abstract" in article:
            abstract = article.get("abstract", "")
            ninjs["description_html"] = abstract
            ninjs["description_text"] = text_utils.get_text(abstract)
        elif article.get("description_text"):
            ninjs["description_text"] = article.get("description_text")

        if article.get("company_codes"):
            ninjs["organisation"] = [
                {
                    "name": c.get("name", ""),
                    "rel": "Securities Identifier",
                    "symbols": [{"ticker": c.get("qcode", ""), "exchange": c.get("security_exchange", "")}],
                }
                for c in article["company_codes"]
            ]
        elif "company" in article:
            ninjs["organisation"] = [{"name": article["company"]}]

        if article.get("rewrite_of"):
            ninjs["evolvedfrom"] = article["rewrite_of"]

        if not ninjs.get("copyrightholder") and not ninjs.get("copyrightnotice") and not ninjs.get("usageterms"):
            ninjs.update(superdesk.get_resource_service("vocabularies").get_rightsinfo(article))

        if article.get("genre"):
            ninjs["genre"] = self._get_genre(article)

        if article.get("flags", {}).get("marked_for_legal"):
            ninjs["signal"] = self._format_signal_cwarn()

        if article.get("attachments"):
            ninjs["attachments"] = self._format_attachments(article)

        if ninjs["type"] == CONTENT_TYPE.TEXT and ("body_html" in ninjs or "body_text" in ninjs):
            if "body_html" in ninjs:
                body_html = ninjs["body_html"]
                word_count = text_utils.get_word_count(body_html)
                char_count = text_utils.get_char_count(body_html)
                readtime = text_utils.get_reading_time(body_html, word_count, article.get("language"))
            else:
                body_text = ninjs["body_text"]
                word_count = text_utils.get_text_word_count(body_text)
                char_count = len(body_text)
                readtime = text_utils.get_reading_time(body_text, word_count, article.get("language"))
            ninjs["charcount"] = char_count
            ninjs["wordcount"] = word_count
            ninjs["readtime"] = readtime

        if article.get("authors"):
            ninjs["authors"] = self._format_authors(article)

        if (article.get("schedule_settings") or {}).get("utc_publish_schedule"):
            ninjs["publish_schedule"] = article["schedule_settings"]["utc_publish_schedule"]

        return ninjs

    def _generate_renditions(self, article):
        """
        For associated items that have custom structure generate renditions based on the item `custom properties.
        """
        renditions = {"original": {}}
        for orig_field, dest_field in self.vidible_fields.items():
            if orig_field in article:
                renditions["original"][dest_field] = article[orig_field]
        if "thumbnail" in article:
            renditions["thumbnail"] = {"href": article["thumbnail"]}
        return renditions

    def can_format(self, format_type, article):
        return format_type == self.format_type

    def _get_type(self, article):
        if article[ITEM_TYPE] == CONTENT_TYPE.PREFORMATTED:
            return CONTENT_TYPE.TEXT
        return article[ITEM_TYPE]

    def _get_associations(self, article, subscriber):
        """Create associations dict for package groups."""
        associations = dict()
        for group in article.get(GROUPS, []):
            if group[GROUP_ID] == ROOT_GROUP:
                continue

            group_items = []
            for ref in group[REFS]:
                if RESIDREF in ref:
                    item = {}
                    item["guid"] = ref[RESIDREF]
                    item[ITEM_TYPE] = ref.get(ITEM_TYPE, "text")
                    if "label" in ref:
                        item["label"] = ref.get("label")
                    if ref.get("package_item"):
                        item.update(self._transform_to_ninjs(ref["package_item"], subscriber, recursive=False))
                    group_items.append(item)
            if len(group_items) == 1:
                associations[group[GROUP_ID]] = group_items[0]
            elif len(group_items) > 1:
                for index in range(0, len(group_items)):
                    associations[group[GROUP_ID] + "-" + str(index)] = group_items[index]
        return associations

    def _format_related(self, article, subscriber):
        """Format all associated items for simple items (not packages)."""
        associations = OrderedDict()
        extra_items = {}
        media = {}
        content_profile = None
        archive_service = superdesk.get_resource_service("archive")

        article_associations = OrderedDict(
            sorted(article.get(ASSOCIATIONS, {}).items(), key=lambda itm: (itm[1] or {}).get("order", 1))
        )

        for key, item in article_associations.items():
            if item:
                if is_related_content(key) and "_type" not in item:
                    orig_item = archive_service.find_one(req=None, _id=item["_id"])
                    orig_item["order"] = item.get("order", 1)
                    item = orig_item.copy()

                item = self._transform_to_ninjs(item, subscriber, recursive=False)

                # Keep original POI and get rid of all other POI.
                renditions = item.get("renditions")
                if renditions:
                    for rendition in renditions.keys():
                        if rendition != "original" and renditions.get(rendition, {}).get("poi"):
                            renditions[rendition].pop("poi", None)

                associations[key] = item  # all items should stay in associations
                match = MEDIA_FIELD_RE.match(key)
                if match:
                    # item id seems to be build from a custom id
                    # we now check content profile to see if it correspond to a custom field
                    if content_profile is None:
                        try:
                            profile = article["profile"]
                        except KeyError:
                            logger.warning("missing profile in article (guid: {guid})".format(guid=article.get("guid")))
                            content_profile = {"schema": {}}
                        else:
                            content_profile = superdesk.get_resource_service("content_types").find_one(
                                _id=profile, req=None
                            )
                    field_id = match.group("field_id")
                    schema = content_profile["schema"].get(field_id, {})
                    if schema.get("type") == "media" or schema.get("type") == "related_content":
                        # we want custom media fields in "extra_items", cf. SDESK-2955
                        version = match.group("version")
                        media.setdefault(field_id, []).append((version, item))
                        extra_items[field_id] = {"type": schema.get("type")}

        if media:
            # we have custom media fields, we now order them
            # and add them to "extra_items"
            for field_id, data in media.items():
                default_order = 1
                items_to_sort = [d[1] for d in sorted(data)]

                if extra_items[field_id]["type"] == "media":
                    # for media items default order is 0 and for related-content default order is 1
                    default_order = 0

                extra_items[field_id]["items"] = sorted(
                    items_to_sort, key=lambda item: item.get("order", default_order)
                )
        return associations, extra_items

    def _get_genre(self, article):
        lang = article.get("language", "")
        return [format_cv_item(item, lang) for item in article["genre"]]

    def _get_subject(self, article):
        """Get subject list for article."""
        return [format_cv_item(item, article.get("language", "")) for item in article.get("subject", [])]

    def _get_service(self, article):
        """Get service list for article.

        It's using `anpa_category` to populate service field for now.
        """
        return [format_cv_item(item, article.get("language", "")) for item in article.get("anpa_category", [])]

    def _get_renditions(self, article):
        """Get renditions for article."""
        # get the actual article's renditions
        actual_renditions = article.get("renditions", {})
        # renditions list that we want to publish
        if article["type"] == "picture":
            renditions_to_publish = self.internal_renditions + list(
                get_renditions_spec(without_internal_renditions=True).keys()
            )
            # filter renditions and keep only the ones we want to publish
            actual_renditions = {
                name: actual_renditions[name] for name in renditions_to_publish if name in actual_renditions
            }
        # format renditions to Ninjs
        renditions = {}
        for name, rendition in actual_renditions.items():
            if rendition:
                renditions[name] = self._format_rendition(rendition)
        return renditions

    def _format_rendition(self, rendition):
        """Format single rendition using fields whitelist."""
        formatted = {}
        for field in self.rendition_properties:
            if field not in rendition:
                continue
            formatted[field] = rendition[field]
            if field in ("width", "height"):
                formatted[field] = int(rendition[field])
        return formatted

    def _format_place(self, article):
        vocabularies_service = superdesk.get_resource_service("vocabularies")
        locator_map = vocabularies_service.find_one(req=None, _id="locators")
        if locator_map and "items" in locator_map:
            locator_map["items"] = vocabularies_service.get_locale_vocabulary(
                locator_map.get("items"), article.get("language")
            )

        def get_label(item):
            if locator_map:
                locators = [loc for loc in locator_map.get("items", []) if loc["qcode"] == item.get("qcode")]
                if locators and len(locators) == 1:
                    return (
                        locators[0].get("state")
                        or locators[0].get("country")
                        or locators[0].get("world_region")
                        or locators[0].get("group")
                    )
            return item.get("name")

        places = []
        for item in article.get("place", []):
            if item.get("scheme") == "geonames":
                places.append(self._format_geonames(item))
            else:
                if config.NINJS_PLACE_EXTENDED:
                    place = {}
                    for key in item.keys():
                        if item.get(key):
                            if key == "qcode":
                                place["code"] = item.get(key)
                            elif key == "name":
                                if get_label(item) is not None:
                                    place["name"] = get_label(item)
                                else:
                                    place["name"] = item.get(key)
                            else:
                                place[key] = item.get(key)
                else:
                    place = {"name": get_label(item), "code": item.get("qcode")}
                places.append(place)
        return places

    def _format_geonames(self, place):
        fields = ["scheme", "code", "name"]
        if app.config.get("NINJS_PLACE_EXTENDED"):
            fields.extend(
                [
                    "state",
                    "state_code",
                    "country",
                    "country_code",
                ]
            )
        geo = {k: v for k, v in place.items() if k in fields}
        if app.config.get("NINJS_PLACE_EXTENDED") and place.get("location"):
            geo["geometry_point"] = {
                "type": "Point",
                "coordinates": [place["location"].get("lat"), place["location"].get("lon")],
            }
        return geo

    def _format_profile(self, profile):
        return superdesk.get_resource_service("content_types").get_output_name(profile)

    def _format_signal_cwarn(self):
        return [{"name": "Content Warning", "code": "cwarn", "scheme": "http://cv.iptc.org/newscodes/signal/"}]

    def _format_attachments(self, article):
        output = []
        attachments_service = superdesk.get_resource_service("attachments")
        for attachment_ref in article["attachments"]:
            attachment = attachments_service.find_one(req=None, _id=attachment_ref["attachment"])
            if superdesk.attachments.is_attachment_public(attachment):  # don't save internal attachments
                output.append(
                    {
                        "id": str(attachment["_id"]),
                        "title": attachment["title"],
                        "description": attachment["description"],
                        "filename": attachment["filename"],
                        "mimetype": attachment["mimetype"],
                        "length": attachment.get("length"),
                        "media": str(attachment["media"]),
                        "href": "/assets/{}".format(str(attachment["media"])),
                    }
                )
        return output

    def _format_authors(self, article):
        users_service = superdesk.get_resource_service("users")
        vocabularies_service = superdesk.get_resource_service("vocabularies")
        job_titles_voc = vocabularies_service.find_one(None, _id="job_titles")
        if job_titles_voc and "items" in job_titles_voc:
            job_titles_voc["items"] = vocabularies_service.get_locale_vocabulary(
                job_titles_voc.get("items"), article.get("language")
            )
        job_titles_map = {v["qcode"]: v["name"] for v in job_titles_voc["items"]} if job_titles_voc is not None else {}

        authors = []
        for author in article["authors"]:
            try:
                user_id = author["parent"]
            except KeyError:
                # XXX: in some older items, parent may be missing, we try to find user with name in this case
                try:
                    user = next(users_service.find({"display_name": author["name"]}))
                except (StopIteration, KeyError):
                    logger.warning("unknown user")
                    user = {}
            else:
                try:
                    user = next(users_service.find({"_id": user_id}))
                except StopIteration:
                    logger.warning("unknown user: {user_id}".format(user_id=user_id))
                    user = {}

            avatar_url = user.get("picture_url", author.get("avatar_url"))

            author = {
                "code": str(user.get("_id", author.get("name", ""))),
                "name": user.get("display_name", author.get("name", "")),
                "role": author.get("role", ""),
                "biography": user.get("biography", author.get("biography", "")),
            }

            # include socials only if they are non-empty
            socials = ("facebook", "twitter", "instagram")
            for social in socials:
                social_data = user.get(social, author.get(social, ""))
                if social_data:
                    author[social] = social_data

            if avatar_url:
                author["avatar_url"] = avatar_url

            job_title_qcode = user.get("job_title")
            if job_title_qcode is not None:
                author["jobtitle"] = {"qcode": job_title_qcode, "name": job_titles_map.get(job_title_qcode, "")}

            authors.append(author)
        return authors

    def export(self, item):
        if self.can_format(self.format_type, item):
            sequence, formatted_doc = self.format(item, {"_id": "0"}, None)[0]
            return formatted_doc.replace("''", "'")
        else:
            raise Exception()


class NINJS2Formatter(NINJSFormatter):
    """NINJS formatter v2

    .. versionadded:: 2.0

    Extending :py:class:`NINJSFormatter` to avoid breaking changes.

    *Changes*:

    - user ``correction_sequence`` for ``version`` field, so it's 1, 2, 3, ... in the output
    - add ``rewrite_sequence`` field
    - add ``rewrite_of`` field

    """

    direct_copy_properties = NINJSFormatter.direct_copy_properties + (
        "rewrite_sequence",
        "rewrite_of",
    )

    def __init__(self):
        super().__init__()
        self.format_type = "ninjs2"

    def _transform_to_ninjs(self, article, subscriber, recursive=True):
        ninjs = super()._transform_to_ninjs(article, subscriber, recursive)
        ninjs["version"] = str(article.get("correction_sequence", 1))
        return ninjs



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import os
import unittest
from pytz import utc

from superdesk.etree import etree
from superdesk.io.feed_parsers.afp_newsml_1_2 import AFPNewsMLOneFeedParser


class TestCase(unittest.TestCase):
    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.join(dirname, "../fixtures", "afp.xml")
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            self.item = AFPNewsMLOneFeedParser().parse(etree.fromstring(f.read()), provider)

    def test_headline(self):
        self.assertEquals(self.item.get("headline"), "Sweden court accepts receivership for Saab carmaker")

    def test_dateline(self):
        self.assertEquals(self.item.get("dateline", {}).get("text"), "STOCKHOLM, Aug 29, 2014 (AFP) -")

    def test_slugline(self):
        self.assertEquals(self.item.get("slugline"), "Sweden-SAAB")

    def test_byline(self):
        self.assertEquals(self.item.get("byline"), "")

    def test_language(self):
        self.assertEquals(self.item.get("language"), "en")

    def test_guid(self):
        self.assertEquals(self.item.get("guid"), "urn:newsml:afp.com:20140829T135002Z:TX-PAR-FXW86:1")

    def test_coreitemvalues(self):
        self.assertEquals(self.item.get("type"), "text")
        self.assertEquals(self.item.get("urgency"), 4)
        self.assertEquals(self.item.get("version"), "1")
        self.assertEquals(self.item.get("versioncreated"), utc.localize(datetime.datetime(2014, 8, 29, 13, 49, 51)))
        self.assertEquals(self.item.get("firstcreated"), utc.localize(datetime.datetime(2014, 8, 29, 13, 49, 51)))
        self.assertEquals(self.item.get("pubstatus"), "usable")

    def test_subjects(self):
        self.assertEquals(len(self.item.get("subject")), 5)
        self.assertIn({"name": "automotive equipment", "qcode": "04011002"}, self.item.get("subject"))
        self.assertIn({"name": "bankruptcy", "qcode": "04016007"}, self.item.get("subject"))
        self.assertIn({"name": "economy, business and finance", "qcode": "04000000"}, self.item.get("subject"))
        self.assertIn(
            {"name": "quarterly or semiannual financial statement", "qcode": "04016038"}, self.item.get("subject")
        )
        self.assertIn({"name": "manufacturing and engineering", "qcode": "04011000"}, self.item.get("subject"))

    def test_usageterms(self):
        self.assertEquals(self.item.get("usageterms"), "NO ARCHIVAL USE")

    def test_genre(self):
        self.assertIn({"name": "business"}, self.item.get("genre"))
        self.assertIn({"name": "bankruptcy"}, self.item.get("genre"))

    def test_content_is_text(self):
        self.assertIsInstance(self.item.get("body_html"), type(""))
        self.assertNotRegex(self.item.get("body_html"), "<body.content>")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from nose.tools import raises
from superdesk.tests import TestCase
from superdesk.privilege import privilege, get_privilege_list, _privileges


class PrivilegeTestCase(TestCase):
    def setUp(self):
        _privileges_saved = _privileges.copy()
        _privileges.clear()

        def revert():
            _privileges.clear()
            _privileges.update(_privileges_saved)

        self.addCleanup(revert)

    def test_privilege_registration(self):
        _privileges.clear()

        privilege(name="ingest", label="Ingest")
        privilege(name="archive", label="Archive")

        self.assertIn("ingest", _privileges)
        self.assertIn("archive", _privileges)

        self.assertEqual(2, len(get_privilege_list()))

    @raises(Exception)
    def test_privilege_name_has_no_dots(self):
        privilege(name="test.")



from behave import then
from steps import json


@then("we get users")
def then_we_get_users(context):
    usernames = json.loads(context.text)
    response_users = json.loads(context.response.get_data())["_items"]
    for i in range(len(usernames)):
        username = usernames[i]
        response_username = response_users[i]["username"]
        assert username == response_username, "user #{} should be {}, but it was {}".format(
            i, username, response_username
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from content_api.tests import ApiTestCase


class PublicApiErrorTestCase(ApiTestCase):
    def _make_one(self, *args, **kwargs):
        """Create and return a new instance of the class under test.

        Make the test fail immediately if the class cannot be imported.
        """
        try:
            from content_api.errors import PublicApiError
        except ImportError:
            self.fail("Could not import class under test")
        else:
            return PublicApiError(*args, **kwargs)

    def test_inherits_from_superdesk_error(self):
        from superdesk.errors import SuperdeskError

        error = self._make_one()
        self.assertIsInstance(error, SuperdeskError)

    def test_uses_given_error_code(self):
        error = self._make_one(12345)
        self.assertEqual(error.status_code, 12345)

    def test_uses_default_error_code_if_error_code_not_given(self):
        error = self._make_one()
        self.assertEqual(error.status_code, 10000)

    def test_uses_default_error_message_if_error_code_not_given(self):
        error = self._make_one()
        self.assertEqual(error.message, "Unknown API error.")

    def test_uses_error_description_if_given(self):
        error = self._make_one(desc="Detailed description")
        self.assertEqual(error.payload, "Detailed description")


class UnknownParameterErrorTestCase(ApiTestCase):
    def _make_one(self, *args, **kwargs):
        """Create and return a new instance of the class under test.

        Make the test fail immediately if the class cannot be imported.
        """
        try:
            from content_api.errors import UnexpectedParameterError
        except ImportError:
            self.fail("Could not import class under test")
        else:
            return UnexpectedParameterError(*args, **kwargs)

    def test_inherits_from_base_publicapi_error(self):
        from content_api.errors import PublicApiError

        error = self._make_one()
        self.assertIsInstance(error, PublicApiError)

    def test_error_code(self):
        error = self._make_one()
        self.assertEqual(error.status_code, 400)

    def test_error_message(self):
        error = self._make_one()
        self.assertEqual(error.message, "Unexpected parameter.")

    def test_uses_error_description_if_given(self):
        error = self._make_one(desc="More detailed description")
        self.assertEqual(error.payload, "More detailed description")


class BadParameterValueTestCase(ApiTestCase):
    def _make_one(self, *args, **kwargs):
        """Create and return a new instance of the class under test.

        Make the test fail immediately if the class cannot be imported.
        """
        try:
            from content_api.errors import BadParameterValueError
        except ImportError:
            self.fail("Could not import class under test")
        else:
            return BadParameterValueError(*args, **kwargs)

    def test_inherits_from_base_publicapi_error(self):
        from content_api.errors import PublicApiError

        error = self._make_one()
        self.assertIsInstance(error, PublicApiError)

    def test_error_code(self):
        error = self._make_one()
        self.assertEqual(error.status_code, 400)

    def test_error_message(self):
        error = self._make_one()
        self.assertEqual(error.message, "Bad parameter value.")

    def test_uses_error_description_if_given(self):
        error = self._make_one(desc="Integer expected for max results")
        self.assertEqual(error.payload, "Integer expected for max results")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX


class CompaniesResource(Resource):
    """
    Company schema
    """

    schema = {
        "name": {"type": "string", "unique": True, "required": True},
        "sd_subscriber_id": {"type": "string"},
        "is_enabled": {"type": "boolean", "default": True},
        "contact_name": {"type": "string"},
        "phone": {"type": "string"},
        "country": {"type": "string"},
    }
    datasource = {"source": "companies", "default_sort": [("name", 1)]}
    item_methods = ["GET", "PATCH", "PUT"]
    resource_methods = ["GET", "POST"]
    mongo_prefix = MONGO_PREFIX



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from eve.versioning import versioned_id_field
from flask import g, current_app as app
from eve.utils import config, ParsedRequest
from .resource import LEGAL_ARCHIVE_NAME
from superdesk import Service, get_resource_privileges
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.item import ITEM_TYPE, GUID_FIELD, CONTENT_TYPE
from superdesk.metadata.packages import GROUPS, RESIDREF, REFS
from superdesk.utils import ListCursor
from superdesk.mongo import set_mongo_lang

logger = logging.getLogger(__name__)


class LegalService(Service):
    """
    Base Service Class for Legal Archive related services
    """

    def on_create(self, docs):
        """Runs on create of legal item.

        Overriding to replace the location of each item in the package to legal archive instead of archive,
        if doc is a package.
        """

        super().on_create(docs)
        for doc in docs:
            if ITEM_TYPE in doc:
                doc.setdefault(config.ID_FIELD, doc.get(GUID_FIELD))
                if doc[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                    self._change_location_of_items_in_package(doc)
            set_mongo_lang(doc)

    def on_replace(self, document, original):
        """Runs on replace of legal item.

        Overriding to replace the location of each item in the package to legal archive instead of archive,
        if doc is a package.
        """

        super().on_replace(document, original)
        if document.get(ITEM_TYPE) == CONTENT_TYPE.COMPOSITE:
            self._change_location_of_items_in_package(document)

    def get(self, req, lookup):
        """Runs on get of legal item.

        Overriding to check if user is authorized to perform get operation on Legal Archive resources. If authorized
        then request is forwarded otherwise throws forbidden error.

        :return: list of docs matching query in req and lookup
        :raises: SuperdeskApiError.forbiddenError() if user is unauthorized to access the Legal Archive resources.
        """

        self.check_get_access_privilege()

        if req and req.args.get("query"):
            lookup["$text"] = {"$search": req.args["query"]}

        return super().get(req, lookup)

    def find_one(self, req, **lookup):
        """Runs on find_one

        Overriding to check if user is authorized to perform get operation on Legal Archive resources. If authorized
        then request is forwarded otherwise throws forbidden error.

        :return: doc if there is one matching the query in req and lookup
        :raises: SuperdeskApiError.forbiddenError() if user is unauthorized to access the Legal Archive resources.
        """

        self.check_get_access_privilege()
        return super().find_one(req, **lookup)

    def check_get_access_privilege(self):
        """Checks if user is authorized to perform get operation on Legal Archive resources.

        If authorized then request is
        forwarded otherwise throws forbidden error.

        :raises: SuperdeskApiError.forbiddenError() if user is unauthorized to access the Legal Archive resources.
        """

        if not hasattr(g, "user"):
            return

        privileges = g.user.get("active_privileges", {})
        resource_privileges = get_resource_privileges(self.datasource).get("GET", None)
        if privileges.get(resource_privileges, 0) == 0:
            raise SuperdeskApiError.forbiddenError()

    def enhance(self, legal_archive_docs):
        """Enhances the item in Legal Archive Service

        :param legal_archive_docs:
        """

        if isinstance(legal_archive_docs, list):
            for legal_archive_doc in legal_archive_docs:
                legal_archive_doc["_type"] = LEGAL_ARCHIVE_NAME
        else:
            legal_archive_docs["_type"] = LEGAL_ARCHIVE_NAME

    def _change_location_of_items_in_package(self, package):
        """
        Changes location of each item in the package to legal archive instead of archive.
        """

        for group in package.get(GROUPS, []):
            for ref in group.get(REFS, []):
                if RESIDREF in ref:
                    ref["location"] = LEGAL_ARCHIVE_NAME


class LegalArchiveService(LegalService):
    def on_fetched(self, docs):
        """
        Overriding this to enhance the published article with the one in archive collection
        """

        self.enhance(docs[config.ITEMS])

    def on_fetched_item(self, doc):
        """
        Overriding this to enhance the published article with the one in archive collection
        """

        self.enhance(doc)


class LegalPublishQueueService(LegalService):
    def create(self, docs, **kwargs):
        """Runs on create of legal publish queue.

        Overriding this from preventing the transmission details again. This happens when an item in a package expires
        at later point of time. In this case, the call to insert transmission details happens twice once when the
        package expires and once when the item expires.
        """

        ids = []
        for doc in docs:
            doc_if_exists = None
            if doc.get(config.ID_FIELD):
                doc_if_exists = self.find_one(req=None, _id=doc.get(config.ID_FIELD))
            if doc_if_exists is None:
                ids.extend(super().create([doc]))

        return ids


class LegalArchiveVersionsService(LegalService):
    def create(self, docs, **kwargs):
        """
        Overriding this from preventing the same version again. This happens when an item is published more than once.
        """

        ids = []
        for doc in docs:
            doc_if_exists = None

            if config.ID_FIELD in doc:  # This happens when inserting docs from pre-populate command
                doc_if_exists = self.find_one(req=None, _id=doc["_id"])

            # This also happens when inserting docs from pre-populate command
            if not doc.get("operation"):
                doc["operation"] = "create"

            if doc_if_exists is None:
                ids.extend(super().create([doc]))

        return ids

    def get(self, req, lookup):
        """
        Version of an article in Legal Archive isn't maintained by Eve. Overriding this to fetch the version history.
        """

        resource_def = app.config["DOMAIN"][LEGAL_ARCHIVE_NAME]
        id_field = versioned_id_field(resource_def)

        if req and req.args and req.args.get(config.ID_FIELD):
            version_history = list(
                super().get_from_mongo(req=ParsedRequest(), lookup={id_field: req.args.get(config.ID_FIELD)})
            )
        else:
            version_history = list(super().get_from_mongo(req=req, lookup=lookup))

        for doc in version_history:
            doc[config.ID_FIELD] = doc[id_field]
            self.enhance(doc)

        return ListCursor(version_history)


class LegalArchiveHistoryService(LegalService):
    def create(self, docs, **kwargs):
        """
        Overriding this from preventing the same version again. This happens when an item is published more than once.
        """

        ids = []
        for doc in docs:
            doc_if_exists = None
            if doc.get("item_id"):
                doc_if_exists = self.find_one(req=None, _id=doc.get("_id"))
            if doc_if_exists is None:
                ids.extend(super().create([doc]))

        return ids



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service
from os.path import dirname, join
from superdesk.tests import TestCase
from superdesk.utc import utcnow
from apps.auth.db.commands import CreateUserCommand, ImportUsersCommand


class CreateUserCommandTestCase(TestCase):
    def test_create_user_command(self):
        if not self.app.config.get("LDAP_SERVER"):
            user = {"username": "foo", "password": "bar", "email": "baz", "password_changed_on": utcnow()}
            cmd = CreateUserCommand()
            cmd.run(user["username"], user["password"], user["email"], admin=True)
            auth_user = get_resource_service("auth_db").authenticate(user)
            self.assertEquals(auth_user["username"], user["username"])

            cmd.run(user["username"], user["password"], user["email"], admin=True)
            auth_user2 = get_resource_service("auth_db").authenticate(user)
            self.assertEquals(auth_user2["username"], user["username"])
            self.assertEquals(auth_user2["_id"], auth_user["_id"])

    def test_create_user_command_no_update(self):
        if not self.app.config.get("LDAP_SERVER"):
            user = {"username": "foo", "password": "bar", "email": "baz", "password_changed_on": utcnow()}
            cmd = CreateUserCommand()
            cmd.run(user["username"], user["password"], user["email"], admin=True)
            cmd.run(user["username"], "new_password", user["email"], admin=True)
            get_resource_service("auth_db").authenticate(user)

    def test_import_users(self):
        """users:import is working with JSON files"""
        roles = [
            {"name": "Writer", "privileges": {}},
            {"name": "Admin", "privileges": {}},
        ]

        self.app.data.insert("roles", roles)

        cmd = ImportUsersCommand()
        fixtures_path = join(dirname(__file__), "fixtures")
        import_file = join(fixtures_path, "import_users_test.json")
        cmd.run(None, import_file)
        users_service = get_resource_service("users")
        roles_services = get_resource_service("roles")

        found_user = users_service.find_one(req=None, username="toto")
        self.assertIsNotNone(found_user)
        role_id = roles_services.find_one(req=None, name="Writer")["_id"]
        self.assertEqual(found_user["role"], role_id)

        found_user = users_service.find_one(req=None, username="titi")
        self.assertIsNotNone(found_user)
        role_id = roles_services.find_one(req=None, name="Admin")["_id"]
        self.assertEqual(found_user["role"], role_id)

        found_user = users_service.find_one(req=None, username="invalid_no_email")
        self.assertIsNone(found_user)

        found_user = users_service.find_one(req=None, username="invalid_unknown_role")
        self.assertIsNone(found_user)

    def test_import_users_csv(self):
        """users:import is working with CSV files"""
        roles = [
            {"name": "Writer", "privileges": {}},
            {"name": "Admin", "privileges": {}},
        ]

        self.app.data.insert("roles", roles)

        cmd = ImportUsersCommand()
        fixtures_path = join(dirname(__file__), "fixtures")
        import_file = join(fixtures_path, "import_users_test.csv")
        cmd.run(["username", "first_name", "last_name", "sign_off", "email", "role"], import_file)
        users_service = get_resource_service("users")
        roles_services = get_resource_service("roles")

        found_user = users_service.find_one(req=None, username="mr_x")
        self.assertIsNotNone(found_user)
        role_id = roles_services.find_one(req=None, name="Writer")["_id"]
        self.assertEqual(found_user["role"], role_id)

        found_user = users_service.find_one(req=None, username="ms_x")
        self.assertIsNotNone(found_user)
        role_id = roles_services.find_one(req=None, name="Admin")["_id"]
        self.assertEqual(found_user["role"], role_id)

    def test_import_users_no_activation_email(self):
        """users:import does not send activation email"""
        roles = [
            {"name": "Writer", "privileges": {}},
            {"name": "Admin", "privileges": {}},
        ]

        self.app.data.insert("roles", roles)

        cmd = ImportUsersCommand()
        fixtures_path = join(dirname(__file__), "fixtures")
        import_file = join(fixtures_path, "import_users_test.json")

        with self.app.app_context():
            with self.app.mail.record_messages() as outbox:
                assert len(outbox) == 0
                cmd.run(None, import_file)
                assert len(outbox) == 0

    def test_import_users_activation_email(self):
        """users:import sends activation link"""
        roles = [
            {"name": "Writer", "privileges": {}},
            {"name": "Admin", "privileges": {}},
        ]

        self.app.data.insert("roles", roles)

        cmd = ImportUsersCommand()
        fixtures_path = join(dirname(__file__), "fixtures")
        import_file = join(fixtures_path, "import_users_test.json")

        with self.app.app_context():
            with self.app.mail.record_messages() as outbox:
                assert len(outbox) == 0
                cmd.run(None, import_file, activation_email=True)
                assert len(outbox) == 2



import unittest

from apps.spellcheck.spellcheck import norvig_suggest


class SpellcheckTestCase(unittest.TestCase):
    def test_normal_suggestions(self):
        """Test case for getting the suggestions for the word which aren't name"""
        model = {"foe": 3, "fox": 5}
        suggestions_1 = norvig_suggest("foo", model)
        self.assertEquals(["fox", "Fox", "foe", "Foe"], suggestions_1)

        suggestions_2 = norvig_suggest("Foo", model)
        self.assertEquals(["fox", "Fox", "foe", "Foe"], suggestions_2)

    def test_name_suggestions(self):
        """Test case for getting the suggestions for name in capitalized case"""

        model = {"Foe": 1, "fox": 5}
        name_suggestion_1 = norvig_suggest("fooe", model)
        self.assertEquals(["Foe"], name_suggestion_1)

        name_suggestion_2 = norvig_suggest("Fooe", model)
        self.assertEquals(["Foe"], name_suggestion_2)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.tests import TestCase


class ExpiredArchiveContentTestCase(TestCase):
    def setUp(self):
        try:
            from apps.archive.commands import RemoveExpiredContent
        except ImportError:
            self.fail("Could not import class under test (RemoveExpiredContent).")
        else:
            self.class_under_test = RemoveExpiredContent
            self.published_items = [
                {
                    "_id": "item1",
                    "item_id": "item1",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "_current_version": 3,
                    "moved_to_legal": True,
                },
                {
                    "_id": "item2",
                    "item_id": "item2",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "_current_version": 3,
                    "moved_to_legal": True,
                },
                {
                    "_id": "item3",
                    "item_id": "item3",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "_current_version": 3,
                    "moved_to_legal": False,
                },
                {
                    "_id": "item4",
                    "item_id": "item4",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "_current_version": 3,
                    "moved_to_legal": True,
                },
            ]

            self.queue_items = [
                {"_id": "item1", "item_id": "item1", "headline": "headline", "item_version": 3, "moved_to_legal": True},
                {
                    "_id": "item2",
                    "item_id": "item2",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "item_version": 3,
                    "moved_to_legal": True,
                },
                {
                    "_id": "item3",
                    "item_id": "item3",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "item_version": 3,
                    "moved_to_legal": False,
                },
                {
                    "_id": "item4",
                    "item_id": "item4",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "item_version": 3,
                    "moved_to_legal": False,
                },
                {
                    "_id": "item5",
                    "item_id": "item4",
                    "headline": "headline",
                    "source": "aap",
                    "body_html": "test",
                    "item_version": 3,
                    "moved_to_legal": True,
                },
            ]

            self.app.data.insert("published", self.published_items)
            self.app.data.insert("publish_queue", self.queue_items)

    def test_items_moved_to_legal_success(self):
        test_items = dict()
        test_items["item1"] = self.published_items[0]
        test_items["item2"] = self.published_items[1]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertDictEqual(result, {})

    def test_items_moved_to_legal_fail_if_published_item_not_moved(self):
        test_items = dict()
        test_items["item2"] = self.published_items[1]
        test_items["item3"] = self.published_items[2]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertIn("item3", result)

    def test_items_moved_to_legal_fail_if_published_queue_item_not_moved(self):
        test_items = dict()
        test_items["item2"] = self.published_items[1]
        test_items["item3"] = self.published_items[3]
        result = self.class_under_test().check_if_items_imported_to_legal_archive(test_items)
        self.assertIn("item3", result)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from datetime import timedelta
from unittest.mock import patch
from superdesk import get_backend
from superdesk.utc import utcnow
from superdesk.tests import TestCase
from apps.auth.db.reset_password import ResetPasswordService


class UserTokenTestCase(TestCase):
    def setUp(self):
        self.service = ResetPasswordService("tokens", backend=get_backend())

    @patch("apps.auth.db.reset_password.get_random_string", return_value="random")
    def test_store_token(self, get_random_string):
        now = utcnow()
        doc = {"user": "foo", "email": "foo@example.com", "_id": "foo"}
        with patch.object(self.service.backend, "create") as create:
            with patch("apps.auth.db.reset_password.utcnow", return_value=now):
                self.service.store_reset_password_token(doc, doc["email"], 10, doc["_id"])
            create.assert_called_with(
                "tokens",
                [
                    {
                        "user": "foo",
                        "email": "foo@example.com",
                        "_id": "foo",
                        "_created": now,
                        "_updated": now,
                        "expire_time": now + timedelta(days=10),
                        "token": "random",
                    }
                ],
            )



# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Scopes allowed in auth_server"""

from enum import Enum

# please refer to /docs/auth_server.rst for details on scopes

Scope = Enum(
    "Scope",
    ["ARCHIVE_READ", "DESKS_READ", "PLANNING_READ", "CONTACTS_READ", "USERS_READ", "ASSIGNMENTS_READ", "EVENTS_READ"],
)


allowed_scopes = {s.name for s in Scope}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
import logging
from superdesk import get_resource_service
from flask import current_app as app
from apps.archive.archive import SOURCE as ARCHIVE
from apps.archive.common import generate_unique_id_and_name, remove_unwanted, insert_into_versions
from superdesk.metadata.item import GUID_TAG, FAMILY_ID, ITEM_STATE, CONTENT_STATE
from superdesk.metadata.utils import generate_guid

logger = logging.getLogger(__name__)


class AppScaffoldDataCommand(superdesk.Command):
    """Initialize the application with some random data fetched from archived.

    Text archive dump must be restored before running this command as well as ``app:initialize_data``
    command to bootstrap the system with desks/users.

    Example:
    ::

        $ python manage.py app:scaffold_data
        $ python manage.py app:scaffold_data --no-of-stories=50

    """

    option_list = [superdesk.Option("--no-of-stories", "-n", dest="no_of_stories", default="200")]

    def run(self, no_of_stories):
        logger.info("Starting scaffolding")
        no_of_stories = int(no_of_stories)

        desks = get_resource_service("desks").get(None, {})

        for i, desk in enumerate(desks):
            logger.info("Adding items for desk:" + str(desk["_id"]))
            self.ingest_items_for(desk, no_of_stories, i + 1)
        return 0

    def ingest_items_for(self, desk, no_of_stories, skip_index):
        desk_id = desk["_id"]
        stage_id = desk["incoming_stage"]

        bucket_size = min(100, no_of_stories)

        no_of_buckets = len(range(0, no_of_stories, bucket_size))

        for x in range(0, no_of_buckets):
            skip = x * bucket_size * skip_index
            logger.info("Page : {}, skip: {}".format(x + 1, skip))
            cursor = get_resource_service("published").get_from_mongo(None, {})
            cursor.skip(skip)
            cursor.limit(bucket_size)
            items = list(cursor)
            logger.info("Inserting {} items".format(len(items)))
            archive_items = []

            for item in items:
                dest_doc = dict(item)
                new_id = generate_guid(type=GUID_TAG)
                dest_doc[app.config["ID_FIELD"]] = new_id
                dest_doc["guid"] = new_id
                generate_unique_id_and_name(dest_doc)

                dest_doc[app.config["VERSION"]] = 1
                dest_doc[ITEM_STATE] = CONTENT_STATE.FETCHED
                user_id = desk.get("members", [{"user": None}])[0].get("user")
                dest_doc["original_creator"] = user_id
                dest_doc["version_creator"] = user_id

                from apps.tasks import send_to

                send_to(dest_doc, desk_id=desk_id, stage_id=stage_id, user_id=user_id)
                dest_doc[app.config["VERSION"]] = 1  # Above step increments the version and needs to reset
                dest_doc[FAMILY_ID] = item["_id"]

                remove_unwanted(dest_doc)
                archive_items.append(dest_doc)

            get_resource_service(ARCHIVE).post(archive_items)
            for item in archive_items:
                insert_into_versions(id_=item[app.config["ID_FIELD"]])


superdesk.command("app:scaffold_data", AppScaffoldDataCommand())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json
import re
import os
from datetime import timedelta

from eve.utils import ParsedRequest

from superdesk.utc import utcnow
from superdesk import get_resource_service
from superdesk.tests import TestCase
from apps.content_filters.filter_condition.filter_condition_service import FilterConditionService
from apps.content_filters.filter_condition.filter_condition import FilterCondition
from apps.content_filters.filter_condition.filter_condition_operator import FilterConditionOperator
from apps.prepopulate.app_populate import AppPopulateCommand


class FilterConditionTests(TestCase):
    def setUp(self):
        self.req = ParsedRequest()
        with self.app.test_request_context(self.app.config.get("URL_PREFIX")):
            self.articles = [
                {"_id": "1", "urgency": 1, "headline": "story", "state": "fetched"},
                {"_id": "2", "headline": "prtorque", "state": "fetched"},
                {"_id": "3", "urgency": 3, "state": "fetched", "flags": {"marked_for_sms": True}},
                {
                    "_id": "4",
                    "urgency": 4,
                    "state": "fetched",
                    "task": {"desk": "1"},
                    "ingest_provider": "1",
                    "associations": {"featuremedia": {"_id": "123456"}},
                },
                {"_id": "5", "urgency": 2, "state": "fetched", "task": {"desk": "2"}, "priority": 3},
                {
                    "_id": "6",
                    "state": "fetched",
                    "embargo": utcnow(),
                    "schedule_settings": {"utc_embargo": utcnow() + timedelta(minutes=20)},
                },
                {"_id": "7", "genre": [{"name": "Sidebar"}], "state": "fetched"},
                {
                    "_id": "8",
                    "subject": [
                        {"name": "adult education", "qcode": "05001000", "parent": "05000000"},
                        {"name": "high schools", "qcode": "05005003", "parent": "05005000"},
                    ],
                    "state": "fetched",
                },
                {"_id": "9", "state": "fetched", "anpa_category": [{"qcode": "a", "name": "Aus News"}]},
                {
                    "_id": "10",
                    "body_html": "<p>Mention<p>",
                    "embargo": utcnow(),
                    "schedule_settings": {"utc_embargo": utcnow() - timedelta(minutes=20)},
                },
                {"_id": "11", "place": [{"qcode": "NSW", "name": "NSW"}], "state": "fetched"},
                {
                    "_id": "12",
                    "body_html": '<div>&#13;\n&#13;\n<body dir="ltr">&#13;\n<div>&#13;\n'
                    "<span>SDA</span><br/>&#13;\n</div>&#13;\n&#13;\n</body>"
                    "&#13;\n</div>",
                    "embargo": utcnow(),
                },
            ]

            self.app.data.insert("archive", self.articles)

            self.app.data.insert(
                "filter_conditions",
                [{"_id": 1, "field": "headline", "operator": "like", "value": "tor", "name": "test-1"}],
            )
            self.app.data.insert(
                "filter_conditions", [{"_id": 2, "field": "urgency", "operator": "in", "value": "2", "name": "test-2"}]
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 3, "field": "urgency", "operator": "in", "value": "3,4,5", "name": "test-2"}],
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 4, "field": "urgency", "operator": "nin", "value": "1,2,3", "name": "test-2"}],
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 5, "field": "urgency", "operator": "in", "value": "2,5", "name": "test-2"}],
            )
            self.app.data.insert(
                "content_filters", [{"_id": 1, "content_filter": [{"expression": {"fc": [1]}}], "name": "soccer-only"}]
            )

    def _setup_elastic_args(self, elastic_translation, search_type="filter"):
        if search_type == "keyword":
            self.req.args = {"source": json.dumps({"query": {"bool": {"filter": [elastic_translation]}}})}
        elif search_type == "not":
            self.req.args = {"source": json.dumps({"query": {"bool": {"must_not": [elastic_translation]}}})}
        elif search_type == "filter":
            self.req.args = {
                "source": json.dumps({"query": {"filtered": {"filter": {"bool": {"should": [elastic_translation]}}}}})
            }
        elif search_type == "match":
            self.req.args = {
                "source": json.dumps(
                    {
                        "query": {
                            "filtered": {"query": {"bool": {"should": [{"bool": {"must": [elastic_translation]}}]}}}
                        }
                    }
                )
            }
        elif search_type == "exists":
            self.req.args = {"source": json.dumps({"query": {"bool": {"must": [elastic_translation]}}})}

    def test_mongo_using_genre_filter_complete_string(self):
        f = FilterCondition("genre", "in", "Sidebar")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("7", docs[0]["_id"])

    def test_mongo_using_priority_compare_eq_filter(self):
        f = FilterCondition("priority", "eq", "3")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("5", docs[0]["_id"])

    def test_mongo_using_priority_compare_ne_filter(self):
        f = FilterCondition("priority", "ne", "3")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(11, docs.count())
            self.assertTrue("5" not in [d["_id"] for d in docs])

    def test_mongo_using_priority_compare_lte_filter(self):
        f = FilterCondition("priority", "lte", "3")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("5", docs[0]["_id"])

    def test_mongo_using_desk_filter_complete_string(self):
        f = FilterCondition("desk", "in", "1")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("4", docs[0]["_id"])

    def test_mongo_using_desk_filter_nin(self):
        f = FilterCondition("desk", "nin", "1")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(11, docs.count())

    def test_mongo_using_desk_filter_eq(self):
        f = FilterCondition("desk", "eq", "1")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("4", docs[0]["_id"])

    def test_mongo_using_sms_filter_with_is(self):
        f = FilterCondition("sms", "in", "true")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())

    def test_mongo_using_sms_filter_with_eq_comp(self):
        f = FilterCondition("sms", "eq", "true")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())

    def test_mongo_using_embargo_filter_with_eq_comp(self):
        f = FilterCondition("embargo", "eq", "true")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())

    def test_mongo_using_desk_filter_in_list(self):
        f = FilterCondition("desk", "in", "1,2")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(2, docs.count())

    def test_mongo_using_category_filter_complete_string(self):
        f = FilterCondition("anpa_category", "in", "a,i")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("9", docs[0]["_id"])

    def test_mongo_using_subject_filter_complete_string(self):
        f = FilterCondition("subject", "in", "05005003")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("8", docs[0]["_id"])

    def test_mongo_using_subject_filter_complete_string_eq(self):
        f = FilterCondition("subject", "eq", "05005003")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("8", docs[0]["_id"])

    def test_mongo_using_like_filter_complete_string(self):
        f = FilterCondition("headline", "like", "story")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("1", docs[0]["_id"])

    def test_mongo_using_like_filter_complete_string_eq(self):
        f = FilterCondition("headline", "like", "story")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("1", docs[0]["_id"])

    def test_mongo_using_like_filter_partial_string(self):
        f = FilterCondition("headline", "like", "tor")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(2, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)

    def test_mongo_using_startswith_filter(self):
        f = FilterCondition("headline", "startswith", "Sto")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("1", docs[0]["_id"])

    def test_mongo_using_endswith_filter(self):
        f = FilterCondition("headline", "endswith", "Que")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("2", docs[0]["_id"])

    def test_mongo_using_notlike_filter(self):
        f = FilterCondition("headline", "notlike", "Que")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(11, docs.count())
            doc_ids = [d["_id"] for d in docs]
            self.assertTrue("2" not in doc_ids)

    def test_mongo_using_in_filter(self):
        f = FilterCondition("urgency", "in", "3,4")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(2, docs.count())
            self.assertEqual("3", docs[0]["_id"])
            self.assertEqual("4", docs[1]["_id"])

    def test_mongo_using_notin_filter(self):
        f = FilterCondition("urgency", "nin", "2,3,4")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(9, docs.count())
            doc_ids = [d["_id"] for d in docs]
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)

    def test_elastic_using_genre_filter_complete_string(self):
        f = FilterCondition("genre", "in", "Sidebar")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("7" in doc_ids)

    def test_elastic_using_sms_filter(self):
        f = FilterCondition("sms", "in", "true")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_elastic_using_embargo_filter(self):
        f = FilterCondition("embargo", "eq", "true")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("6" in doc_ids)

    def test_elastic_using_subject_filter_complete_string(self):
        f = FilterCondition("subject", "in", "05005003")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("8" in doc_ids)

    def test_elastic_using_anpa_category_filter_complete_string(self):
        f = FilterCondition("anpa_category", "in", "a,i")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("9" in doc_ids)

    def test_elastic_using_in_filter(self):
        f = FilterCondition("urgency", "in", "3,4")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(2, docs.count())
            self.assertTrue("4" in doc_ids)
            self.assertTrue("3" in doc_ids)

    def test_elastic_using_eq_filter(self):
        f = FilterCondition("urgency", "eq", "3")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_elastic_using_nin_filter(self):
        f = FilterCondition("urgency", "nin", "3,4")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, "not")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            self.assertEqual(8, docs.count())
            doc_ids = [d["_id"] for d in docs]
            self.assertTrue("6" in doc_ids)
            self.assertTrue("5" in doc_ids)

    def test_elastic_using_ne_filter(self):
        f = FilterCondition("urgency", "ne", "4")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, "not")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            self.assertEqual(9, docs.count())
            doc_ids = [d["_id"] for d in docs]
            self.assertTrue("6" in doc_ids)
            self.assertTrue("5" in doc_ids)

    def test_elastic_using_like_filter(self):
        f = FilterCondition("headline", "like", "Tor")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, "keyword")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            self.assertEqual(2, docs.count())
            doc_ids = [d["_id"] for d in docs]
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)

    def test_elastic_using_notlike_filter(self):
        f = FilterCondition("headline", "notlike", "que")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, "not")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            self.assertEqual(9, docs.count())
            doc_ids = [d["_id"] for d in docs]
            self.assertTrue("2" not in doc_ids)

    def test_elastic_using_startswith_filter(self):
        f = FilterCondition("headline", "startswith", "Sto")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, "keyword")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            self.assertEqual(1, docs.count())
            self.assertEqual("1", docs[0]["_id"])

    def test_elastic_using_endswith_filter(self):
        f = FilterCondition("headline", "endswith", "Que")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, "keyword")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            self.assertEqual(1, docs.count())
            self.assertEqual("2", docs[0]["_id"])

    def test_get_mongo_operator(self):
        self.assertEqual(FilterConditionOperator.factory("in").mongo_operator, "$in")
        self.assertEqual(FilterConditionOperator.factory("nin").mongo_operator, "$nin")
        self.assertEqual(FilterConditionOperator.factory("like").mongo_operator, "$regex")
        self.assertEqual(FilterConditionOperator.factory("notlike").mongo_operator, "$not")
        self.assertEqual(FilterConditionOperator.factory("startswith").mongo_operator, "$regex")
        self.assertEqual(FilterConditionOperator.factory("endswith").mongo_operator, "$regex")

    def test_get_mongo_value(self):
        f = FilterCondition("urgency", "in", "1,2")
        self.assertEqual(f.value.get_mongo_value(f.field), [1, 2])

        f = FilterCondition("priority", "nin", "3")
        self.assertEqual(f.value.get_mongo_value(f.field), [3])

        f = FilterCondition("headline", "like", "test")
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile(".*test.*", re.IGNORECASE))

        f = FilterCondition("headline", "notlike", "test")
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile(".*test.*", re.IGNORECASE))

        f = FilterCondition("headline", "startswith", "test")
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile("^test", re.IGNORECASE))

        f = FilterCondition("headline", "endswith", "test")
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile(".*test", re.IGNORECASE))

    def test_does_match_with_eq(self):
        f = FilterCondition("urgency", "eq", "1")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_eq_string(self):
        f = FilterCondition("headline", "eq", "Story")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_eq_bool(self):
        f = FilterCondition("sms", "eq", "true")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_lt_int(self):
        f = FilterCondition("priority", "lt", "7")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_gt_int(self):
        f = FilterCondition("priority", "gt", "1")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_gte_int(self):
        f = FilterCondition("priority", "gte", "3")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_ne_int(self):
        f = FilterCondition("priority", "ne", "3")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_like_full(self):
        f = FilterCondition("headline", "like", "story")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_like_partial(self):
        f = FilterCondition("headline", "like", "tor")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_startswith_filter(self):
        f = FilterCondition("headline", "startswith", "Sto")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_startswith_filter_html_field(self):
        f = FilterCondition("body_html", "startswith", "men")
        self.assertTrue(f.does_match(self.articles[9]))

    def test_does_match_with_startswith_filter_html_field_with_fluff(self):
        f = FilterCondition("body_html", "startswith", "SDA")
        self.assertTrue(f.does_match(self.articles[11]))

    def test_does_match_with_endswith_filter(self):
        f = FilterCondition("headline", "endswith", "Que")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_notlike_filter(self):
        f = FilterCondition("headline", "notlike", "Que")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))

    def test_does_match_with_genre_filter(self):
        f = FilterCondition("genre", "in", "Sidebar")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))
        self.assertFalse(f.does_match({"genre": None}))
        self.assertTrue(f.does_match({"genre": [{"name": "Sidebar"}]}))
        self.assertFalse(f.does_match({"genre": [{"name": "Article"}]}))
        self.assertTrue(f.does_match({"genre": [{"name": "Sidebar"}, {"name": "Article"}]}))

    def test_does_match_with_category_filter(self):
        f = FilterCondition("anpa_category", "in", "a,i")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))
        self.assertTrue(f.does_match(self.articles[8]))

    def test_does_match_with_subject_filter(self):
        f = FilterCondition("subject", "in", "05005003")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_sms_filter(self):
        f = FilterCondition("sms", "nin", "true")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_embargo_filter_with_false(self):
        f = FilterCondition("embargo", "eq", "false")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_embargo_filter_with_true(self):
        f = FilterCondition("embargo", "eq", "true")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_in_filter(self):
        f = FilterCondition("urgency", "in", "3,4")
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_in_filter_case_insensitive(self):
        f = FilterCondition("source", "in", "aap,reuters")
        self.assertTrue(f.does_match({"source": "AAP"}))
        self.assertTrue(f.does_match({"source": "aap"}))
        self.assertTrue(f.does_match({"source": "REUTERS"}))
        f = FilterCondition("source", "in", "AAP")
        self.assertTrue(f.does_match({"source": "AAP"}))
        self.assertTrue(f.does_match({"source": "aap"}))
        self.assertFalse(f.does_match({"source": "REUTERS"}))

    def test_does_match_with_nin_filter(self):
        f = FilterCondition("urgency", "nin", "2,3,4")
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))

    def test_are_equal1(self):
        f = FilterConditionService()
        new_doc = {"name": "A", "field": "urgency", "operator": "nin", "value": "2,3,4"}
        doc = {"_id": 1, "name": "B", "field": "urgency", "operator": "nin", "value": "2,3,4"}
        self.assertTrue(f._are_equal(new_doc, doc))

    def test_are_equal2(self):
        f = FilterConditionService()
        new_doc = {"name": "A", "field": "urgency", "operator": "nin", "value": "4,2,3"}
        doc = {"_id": 1, "name": "B", "field": "urgency", "operator": "nin", "value": "2,3,4"}
        self.assertTrue(f._are_equal(new_doc, doc))

    def test_are_equal3(self):
        f = FilterConditionService()
        new_doc = {"name": "A", "field": "urgency", "operator": "nin", "value": "jump,track"}
        doc = {"_id": 1, "name": "B", "field": "urgency", "operator": "nin", "value": "tump,jrack"}
        self.assertTrue(f._are_equal(new_doc, doc))

    def test_are_equal4(self):
        f = FilterConditionService()
        new_doc = {"name": "A", "field": "urgency", "operator": "nin", "value": "4,2,3"}
        doc = {"_id": 1, "name": "B", "field": "urgency", "operator": "nin", "value": "2,3"}
        self.assertFalse(f._are_equal(new_doc, doc))

    def test_if_fc_is_used(self):
        f = FilterConditionService()
        with self.app.app_context():
            self.assertTrue(f._get_referenced_filter_conditions(1).count() == 1)
            self.assertTrue(f._get_referenced_filter_conditions(2).count() == 0)

    def test_check_similar(self):
        f = get_resource_service("filter_conditions")
        filter_condition1 = {"field": "urgency", "operator": "in", "value": "2"}
        filter_condition2 = {"field": "urgency", "operator": "in", "value": "3"}
        filter_condition3 = {"field": "urgency", "operator": "in", "value": "1"}
        filter_condition4 = {"field": "urgency", "operator": "in", "value": "5"}
        filter_condition5 = {"field": "urgency", "operator": "nin", "value": "5"}
        filter_condition6 = {"field": "headline", "operator": "like", "value": "tor"}
        with self.app.app_context():
            cmd = AppPopulateCommand()
            filename = os.path.join(
                os.path.abspath(os.path.dirname("apps/prepopulate/data_init/vocabularies.json")), "vocabularies.json"
            )
            cmd.run(filename)
            self.assertTrue(len(f.check_similar(filter_condition1)) == 2)
            self.assertTrue(len(f.check_similar(filter_condition2)) == 1)
            self.assertTrue(len(f.check_similar(filter_condition3)) == 0)
            self.assertTrue(len(f.check_similar(filter_condition4)) == 3)
            self.assertTrue(len(f.check_similar(filter_condition5)) == 1)
            self.assertTrue(len(f.check_similar(filter_condition6)) == 1)

    def test_mongo_using_place_filter_complete_string(self):
        f = FilterCondition("place", "in", "NSW")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("11", docs[0]["_id"])

    def test_elastic_using_place_filter_complete_string(self):
        f = FilterCondition("place", "match", "NSW")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, "match")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("11" in doc_ids)

    def test_mongo_using_ingest_provider_filter_eq(self):
        f = FilterCondition("ingest_provider", "eq", "1")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("4", docs[0]["_id"])

    def test_elastic_using_ingest_provider_filter_eq(self):
        f = FilterCondition("ingest_provider", "eq", "1")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("4" in doc_ids)

    def test_mongo_featuremedia_exists(self):
        f = FilterCondition("featuremedia", "exists", "true")
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual("4", docs[0]["_id"])

    def test_elastic_using_featuremedia_exists(self):
        f = FilterCondition("featuremedia", "exists", "true")
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, search_type="exists")
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("4" in doc_ids)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk import app
from datetime import timedelta
from superdesk.utc import utcnow
from eve.utils import date_to_str
from superdesk import get_resource_service
import logging

logger = logging.getLogger(__name__)


class RemoveExpiredSessions(superdesk.Command):
    """Remove expired sessions from db.

    Using ``SESSION_EXPIRY_MINUTES`` config.

    Example:
    ::

        $ python manage.py session:gc

    """

    def run(self):
        self.remove_expired_sessions()

    def remove_expired_sessions(self):
        auth_service = get_resource_service("auth")
        expiry_minutes = app.settings["SESSION_EXPIRY_MINUTES"]
        expiration_time = utcnow() - timedelta(minutes=expiry_minutes)
        logger.info("Deleting session not updated since {}".format(expiration_time))
        query = {"_updated": {"$lte": date_to_str(expiration_time)}}
        sessions = auth_service.get(req=None, lookup=query)
        for session in sessions:
            auth_service.delete({"_id": str(session["_id"])})
        self._update_online_users()

    def _update_online_users(self):
        online_users = self._get_online_users()
        active_sessions_ids = self._get_active_session_ids()
        for user in online_users:
            session_preferences = user.get("session_preferences", {})
            active = {_id: data for _id, data in session_preferences.items() if active_sessions_ids.get(_id)}
            if len(active) != len(session_preferences):
                get_resource_service("users").system_update(user["_id"], {"session_preferences": active}, user)

    def _get_active_session_ids(self):
        active_sessions = get_resource_service("auth").get(req=None, lookup={})
        return {str(sess["_id"]): True for sess in active_sessions}

    def _get_online_users(self):
        return get_resource_service("users").get_from_mongo(
            None, {"session_preferences": {"$exists": True, "$nin": [None, {}]}}
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.tests import TestCase

from superdesk.etree import etree
from superdesk.io.feed_parsers import wordpress_wxr
from unittest import mock
import datetime

FAKE_MEDIA_DATA = {
    "_created": datetime.datetime(2017, 4, 26, 13, 0, 33, tzinfo=datetime.timezone.utc),
    "_etag": "0b135dc3959fe4b09642d02ae733b94af72af5d3",
    "_id": "590099f1cc3a2d2349a785f4",
    "_updated": datetime.datetime(2017, 4, 26, 13, 0, 33, tzinfo=datetime.timezone.utc),
    "media": "590099f1cc3a2d2349a785ee",
    "mimetype": "image/jpeg",
    "renditions": {
        "original": {
            "height": 256,
            "href": "http://test",
            "media": "590099f1cc3a2d2349a785ee",
            "mimetype": "image/jpeg",
            "width": 642,
        },
        "thumbnail": {
            "height": 23,
            "href": "http://test",
            "media": "590099f1cc3a2d2349a785f0",
            "mimetype": "image/jpeg",
            "width": 60,
        },
        "viewImage": {
            "height": 79,
            "href": "http://test",
            "media": "590099f1cc3a2d2349a785f2",
            "mimetype": "image/jpeg",
            "width": 200,
        },
    },
}


def fake_update_renditions(item, url, _):
    update = {
        # we use URL as _id so we can check the value easily
        "_id": url,
        "renditions": {
            "original": {
                "height": 256,
                "href": "http://test",
                "media": "590099f1cc3a2d2349a785ee",
                "mimetype": "image/jpeg",
                "width": 642,
            },
            "thumbnail": {
                "height": 23,
                "href": "http://test",
                "media": "590099f1cc3a2d2349a785f0",
                "mimetype": "image/jpeg",
                "width": 60,
            },
            "viewImage": {
                "height": 79,
                "href": "http://test",
                "media": "590099f1cc3a2d2349a785f2",
                "mimetype": "image/jpeg",
                "width": 200,
            },
        },
        "mimetype": "image/jpeg",
    }
    item.update(update)


class WPWXRTestBase(TestCase):
    @mock.patch.object(wordpress_wxr, "update_renditions", fake_update_renditions)
    def __init__(self, methodname):
        super().__init__(methodname)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            buf = f.read()
        self.ori_file = buf
        parser = etree.XMLParser(recover=True)
        parsed = etree.fromstring(buf, parser)
        self.articles = wordpress_wxr.WPWXRFeedParser().parse(parsed, provider)


class WPWXRTestCase(WPWXRTestBase):

    filename = "wordpress_wxr.xml"

    def test_guid(self):
        self.assertEqual(self.articles[0]["guid"], "http://sdnewtester.org/?p=216")

    def test_firstpublished(self):
        self.assertEqual(
            self.articles[0]["firstpublished"], datetime.datetime(2013, 7, 29, 16, 3, 54, tzinfo=datetime.timezone.utc)
        )

    def test_author(self):
        self.assertEqual(self.articles[0]["author"], "admin")

    def test_headline(self):
        self.assertEqual(self.articles[0]["headline"], "Starafrica to dispose assets for $6 million to clear debt")

    def test_body_html(self):
        expected = (
            "<p>By Tester\nHarare, July 19 (The SDNewTester) - "
            "Cash-strapped StarAfrica Corporation is set to dis"
            "pose its transport company and its stake in Tongaa"
            "t Hulett Botswana for $6 million to offset part of"
            " the companyâs $19.7 million debt.\nBla bla test</p>"
        )
        self.assertEqual(self.articles[0]["body_html"], expected)

    def test_body_html_non_void(self):
        """Check that non void elements are not self closing (which is illegal in HTML 5)

        SDESK-3758
        """
        expected = (
            '<hr/><p>\t\t<img src="http://test"/></p>\t\t<iframe src="https://test.invalid"'
            ' width="750" height="400" frameborder="0" allowfullscreen="allowfull'
            'screen"></iframe>'
        )
        self.assertEqual(self.articles[4]["body_html"], expected)

    def test_keywords(self):
        self.assertEqual(self.articles[0]["keywords"], ["companies"])

    def test_category(self):
        self.assertEqual(
            self.articles[0]["anpa_category"],
            [
                {"qcode": "Business", "name": "Business"},
                {"qcode": "Companies", "name": "Companies"},
                {"qcode": "Economy", "name": "Economy"},
            ],
        )

    def test_attachments(self):
        # in self.articles[1] there is a body, so image should be in associations
        expected = {
            "_id": "http://example.net/image.png",
            "guid": "http://example.net/image.png",
            "ingest_provider": "wpwxr",
            "headline": "test2",
            "alt_text": " ",
            "description_text": " ",
            "mimetype": "image/jpeg",
            "renditions": {
                "original": {
                    "height": 256,
                    "href": "http://test",
                    "media": "590099f1cc3a2d2349a785ee",
                    "mimetype": "image/jpeg",
                    "width": 642,
                },
                "thumbnail": {
                    "height": 23,
                    "href": "http://test",
                    "media": "590099f1cc3a2d2349a785f0",
                    "mimetype": "image/jpeg",
                    "width": 60,
                },
                "viewImage": {
                    "height": 79,
                    "href": "http://test",
                    "media": "590099f1cc3a2d2349a785f2",
                    "mimetype": "image/jpeg",
                    "width": 200,
                },
            },
            "type": "picture",
        }
        self.assertEqual(self.articles[1]["associations"]["featuremedia"], expected)

    def test_attachments_no_body(self):
        # in self.articles[1] there is no body, so item should be an image
        # cf. SDTS-29
        expected = {
            "original": {
                "height": 256,
                "href": "http://test",
                "media": "590099f1cc3a2d2349a785ee",
                "mimetype": "image/jpeg",
                "width": 642,
            },
            "thumbnail": {
                "height": 23,
                "href": "http://test",
                "media": "590099f1cc3a2d2349a785f0",
                "mimetype": "image/jpeg",
                "width": 60,
            },
            "viewImage": {
                "height": 79,
                "href": "http://test",
                "media": "590099f1cc3a2d2349a785f2",
                "mimetype": "image/jpeg",
                "width": 200,
            },
        }
        self.assertNotIn("associations", self.articles[3])
        self.assertEqual(self.articles[3]["renditions"], expected)
        self.assertEqual(self.articles[3]["type"], "picture")

    def test_clrf(self):
        expected = (
            "<p>By Tester</p><p>Harare, July 19 (The SDNewTester) - Cash-strapped"
            " StarAfrica Corporation is set to dispose its transport company and "
            "its stake in Tongaat Hulett Botswana for $6 million to offset part o"
            "f the companyâs $19.7 million debt.</p><hr><p>Bla bla test</p>"
        )
        self.assertEqual(self.articles[2]["body_html"], expected)


class WPWXRThumbnailTestCase(WPWXRTestBase):

    filename = "wordpress_wxr_thumb.xml"

    def test_skipped(self):
        """Check that "attachment" items are skipped"""
        # in the test file are 2 items: one "attachment" and one "post",
        # only the "post" item must be returned
        self.assertEqual(len(self.articles), 1)

    def test_thumbnail(self):
        """Check that thumbnail is retrieved from other item and used as feature media (SDESK-3699)"""
        expected = {
            "type": "picture",
            "ingest_provider": "wpwxr",
            "_id": "https://toto.invalid/attachment.jpg",
            "guid": "https://toto.invalid/attachment.jpg",
            "renditions": {
                "original": {
                    "height": 256,
                    "href": "http://test",
                    "media": "590099f1cc3a2d2349a785ee",
                    "mimetype": "image/jpeg",
                    "width": 642,
                },
                "thumbnail": {
                    "height": 23,
                    "href": "http://test",
                    "media": "590099f1cc3a2d2349a785f0",
                    "mimetype": "image/jpeg",
                    "width": 60,
                },
                "viewImage": {
                    "height": 79,
                    "href": "http://test",
                    "media": "590099f1cc3a2d2349a785f2",
                    "mimetype": "image/jpeg",
                    "width": 200,
                },
            },
            "mimetype": "image/jpeg",
            "description_text": "this is a description",
            "alt_text": "test",
            "headline": "test post",
        }

        self.assertEqual(self.articles[0]["associations"]["featuremedia"], expected)


class FunkeWXRTestCase(WPWXRTestBase):

    filename = "wordpress_wxr_funke.xml"

    def test_body_html(self):
        self.assertTrue(
            self.articles[0]["body_html"].lstrip().startswith("<p><strong"), self.articles[0]["body_html"][:20]
        )



from .base import BasePackageElement


class Mimetype(BasePackageElement):
    MIMETYPE = "application/vnd.adobe.indesign-idml-package"

    @property
    def filename(self):
        """
        Filename inside IDML package.
        Used as a filename for a file inside zip container.
        :return str: filename
        """
        return "mimetype"

    def _build_etree(self):
        pass

    def render(self):
        return self.MIMETYPE



"""
Superdesk SAML Authentication

.. versionadded:: 1.9

Configuration
~~~~~~~~~~~~~

To enable SAML auth module, add ``superdesk.auth.saml`` into :ref:`settings.installed_apps` in settings.
Setting :ref:`settings.secret_key` is also required.

Finally you have to specify :ref:`settings.saml_path` in settings where your ``settings.json`` file and
``certs`` are located. See https://github.com/onelogin/python3-saml#how-it-works for more info.

Service provider config for superdesk in ``settings.json`` file example::

    "sp": {
        "entityId": "http://example.com/api/login/saml_metadata",
        "assertionConsumerService": {
            "url": "http://example.com/api/login/saml?acs",
            "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST"
        },
        "singleLogoutService": {
            "url": "http://example.com/api/login/saml?sls",
            "binding": "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
        },
        "NameIDFormat": "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress",
        "x509cert": "",
        "privateKey": ""
    }

"""

import superdesk
import logging

from urllib.parse import urlparse

from flask import current_app as app, request, redirect, make_response, session, jsonify, json
from superdesk.auth import auth_user

try:
    from onelogin.saml2.auth import OneLogin_Saml2_Auth

    imported = True
except ImportError:
    imported = False


SESSION_NAME_ID = "samlNameId"
SESSION_SESSION_ID = "samlSessionIndex"
SESSION_USERDATA_KEY = "samlUserdata"

bp = superdesk.Blueprint("saml", __name__)
logger = logging.getLogger(__name__)


def init_app(app):
    app.client_config["saml_auth"] = False
    if app.config.get("SAML_PATH"):
        assert imported, "onelogin module is not available"
        app.client_config["saml_auth"] = True
        app.client_config["saml_label"] = app.config["SAML_LABEL"]
        superdesk.blueprint(bp, app)


def init_saml_auth(req):
    auth = OneLogin_Saml2_Auth(req, custom_base_path=app.config["SAML_PATH"])
    return auth


def prepare_flask_request(request):
    url_data = urlparse(request.url)
    scheme = request.scheme
    if app.config.get("SERVER_URL"):
        scheme = urlparse(app.config["SERVER_URL"]).scheme or request.scheme
    return {
        "https": "on" if scheme == "https" else "off",
        "http_host": request.host,
        "server_port": url_data.port,
        "script_name": request.path,
        "get_data": request.args.copy(),
        "post_data": request.form.copy(),
    }


USERDATA_MAPPING = {
    "displayname": "display_name",
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/name": "username",
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/givenname": "first_name",
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/surname": "last_name",
    "http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress": "email",
    "http://schemas.xmlsoap.org/claims/Group": "desk",
    "http://schemas.microsoft.com/ws/2008/06/identity/claims/role": "role",
}


def get_userdata(saml_data):
    userdata = {}
    for src, dest in USERDATA_MAPPING.items():
        try:
            userdata[dest] = saml_data[src][0]
        except (KeyError, IndexError):
            continue
    return userdata


@bp.route("/login/saml", methods=["GET", "POST"])
def index():
    req = prepare_flask_request(request)
    auth = init_saml_auth(req)
    errors = []

    if "slo" in request.args:
        name_id = None
        session_index = None
        if SESSION_NAME_ID in session:
            name_id = session[SESSION_NAME_ID]
        if SESSION_SESSION_ID in session:
            session_index = session[SESSION_SESSION_ID]
        return redirect(auth.logout(name_id=name_id, session_index=session_index))
    elif "acs" in request.args or request.form:
        auth.process_response()
        errors = auth.get_errors()
        if len(errors) == 0:
            session[SESSION_NAME_ID] = auth.get_nameid()
            session[SESSION_SESSION_ID] = auth.get_session_index()
            session[SESSION_USERDATA_KEY] = auth.get_attributes()
        else:
            logger.error("SAML %s reason=%s", errors, auth.get_last_error_reason())
            return jsonify(
                {
                    "req": req,
                    "errors": errors,
                    "error_reason": auth.get_last_error_reason(),
                }
            )
    elif "sls" in request.args:

        def dscb():
            session.clear()

        url = auth.process_slo(delete_session_cb=dscb)
        errors = auth.get_errors()
        if len(errors) == 0:
            if url is not None:
                return redirect(url)

    if session.get(SESSION_NAME_ID):
        return auth_user(session[SESSION_NAME_ID], get_userdata(session[SESSION_USERDATA_KEY]))

    return redirect(auth.login())


@bp.route("/login/saml_metadata")
def metadata():
    req = prepare_flask_request(request)
    auth = init_saml_auth(req)
    settings = auth.get_settings()
    metadata = settings.get_sp_metadata()
    errors = settings.validate_metadata(metadata)

    if len(errors) == 0:
        resp = make_response(metadata, 200)
        resp.headers["Content-Type"] = "text/xml"
    else:
        resp = make_response(", ".join(errors), 500)
    return resp



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : petr
# Creation: 2019-08-01 14:52

from bson import ObjectId
from bson.errors import InvalidId
from superdesk import get_resource_service
from superdesk.commands.data_updates import BaseDataUpdate
from superdesk.vocabularies import is_related_content


class DataUpdate(BaseDataUpdate):

    resource = "archive"

    def forwards(self, mongodb_collection, mongodb_database):
        related = list(get_resource_service("vocabularies").get(req=None, lookup={"field_type": "related_content"}))
        archive_service = get_resource_service("archive")
        for resource in ("archive", "published"):
            service = get_resource_service(resource)
            for item in mongodb_database[resource].find({"type": "text", "associations": {"$gt": {}}}):
                update = False
                associations = {}
                for key, val in item["associations"].items():
                    if val and is_related_content(key, related) and len(val.keys()) > 2:
                        update = True
                        associations[key] = {
                            "_id": val["_id"],
                            "type": val.get("type", "text"),
                        }
                    elif val and val.get("_id") and len(val.keys()) == 1:
                        type_ = mongodb_database[resource].find_one({"_id": val["_id"]}, {"type": 1})
                        if type_:
                            update = True
                            associations[key] = val
                            associations[key]["type"] = type_["type"]
                    else:
                        associations[key] = val
                if update:
                    try:
                        _id = ObjectId(item["_id"])
                    except InvalidId:
                        _id = item["_id"]
                    # must update twice, otherwise it merges the changes
                    service.system_update(_id, {"associations": None}, item)
                    service.system_update(_id, {"associations": associations}, item)

    def backwards(self, mongodb_collection, mongodb_database):
        pass



import os
import shutil
from unittest import mock

import superdesk.commands.data_updates
from superdesk import get_resource_service
from superdesk.commands.data_updates import get_data_updates_files, GenerateUpdate, Upgrade, Downgrade, get_dirs
from superdesk.tests import TestCase

# change the folder where to store updates for test purpose
DEFAULT_DATA_UPDATE_DIR_NAME = "/tmp/data_updates"
MAIN_DATA_UPDATES_DIR = "/tmp/global_data_updates"


class DataUpdatesTestCase(TestCase):
    def setUp(self):

        dirs = (
            ("DEFAULT_DATA_UPDATE_DIR_NAME", "/tmp/data_updates"),
            ("MAIN_DATA_UPDATES_DIR", "/tmp/global_data_updates"),
        )
        for name, path in dirs:
            shutil.rmtree(path, True)
            os.mkdir(path)
            self.addCleanup(lambda path=path: shutil.rmtree(path))

            patcher = mock.patch("superdesk.commands.data_updates.%s" % name, path)
            self.addCleanup(patcher.stop)
            patcher.start()

        # update the default implementation for `forwards` and `backwards` function
        for n in ("FW", "BW"):
            name = "DEFAULT_DATA_UPDATE_%s_IMPLEMENTATION" % n
            patcher = mock.patch("superdesk.commands.data_updates.%s" % name, "pass")
            self.addCleanup(patcher.stop)
            patcher.start()

        self.app.config["APPS_DATA_UPDATES_PATHS"] = []

    def test_data_update_generation(self):
        assert len(get_data_updates_files()) == 0, get_data_updates_files()
        GenerateUpdate().run(resource_name="RESOURCE_NAME")
        assert len(get_data_updates_files()) == 1, get_data_updates_files()
        GenerateUpdate().run(resource_name="RESOURNCE_NAME")
        assert len(get_data_updates_files()) == 2, get_data_updates_files()

    def test_data_update_generation_create_updates_dir(self):
        updates_dir = DEFAULT_DATA_UPDATE_DIR_NAME
        shutil.rmtree(DEFAULT_DATA_UPDATE_DIR_NAME)
        self.assertFalse(os.path.exists(updates_dir))
        self.app.config["DATA_UPDATES_PATH"] = updates_dir
        GenerateUpdate().run("tmp")
        self.assertTrue(os.path.exists(updates_dir))

    def number_of_data_updates_applied(self):
        return get_resource_service("data_updates").find({}).count()

    def test_dry_data_update(self):
        superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = """
            count = mongodb_collection.find({}).count()
            assert count == 0, count
        """
        self.assertEqual(self.number_of_data_updates_applied(), 0)
        GenerateUpdate().run(resource_name="data_updates")
        Upgrade().run(dry=True)
        self.assertEqual(self.number_of_data_updates_applied(), 0)

    def test_fake_data_update(self):
        self.assertEqual(self.number_of_data_updates_applied(), 0)
        superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = "raise Exception()"
        superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION = "raise Exception()"
        GenerateUpdate().run(resource_name="data_updates")
        self.assertEqual(self.number_of_data_updates_applied(), 0)
        Upgrade().run(fake=True)
        self.assertEqual(self.number_of_data_updates_applied(), 1)
        Downgrade().run(fake=True)
        self.assertEqual(self.number_of_data_updates_applied(), 0)

    def test_data_update(self):
        # create migrations
        for index in range(40):
            superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = """
            assert mongodb_collection
            count = mongodb_collection.find({}).count()
            assert count == %d, count
            assert mongodb_database
            """ % (
                index
            )
            superdesk.commands.data_updates.DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION = """
            assert mongodb_collection
            count = mongodb_collection.find({}).count()
            assert count == %d, count
            assert mongodb_database
            """ % (
                index + 1
            )
            GenerateUpdate().run(resource_name="data_updates")
        assert self.number_of_data_updates_applied() == 0
        thirdieth_update = get_data_updates_files(True)[29]
        Upgrade().run(data_update_id=thirdieth_update)
        assert self.number_of_data_updates_applied() == 30
        Upgrade().run()
        assert self.number_of_data_updates_applied() == 40
        Downgrade().run()
        assert self.number_of_data_updates_applied() == 39
        Downgrade().run(data_update_id=thirdieth_update)
        assert self.number_of_data_updates_applied() == 29
        Upgrade().run()

    def test_multiple_dirs(self):
        with mock.patch.dict(self.app.config, {"APPS_DATA_UPDATES_PATHS": ["/tmp/foo", "tmp/bar"]}):
            dirs = get_dirs()
            self.assertEqual(4, len(dirs))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from flask_babel import _

logger = logging.getLogger(__name__)


class RuleSetsResource(Resource):
    schema = {
        "name": {
            "type": "string",
            "iunique": True,
            "required": True,
            "nullable": False,
            "empty": False,
            "minlength": 1,
        },
        "rules": {"type": "list"},
    }

    privileges = {"POST": "rule_sets", "DELETE": "rule_sets", "PATCH": "rule_sets"}

    mongo_indexes = {
        "name_1": ([("name", 1)], {"unique": True}),
    }


class RuleSetsService(BaseService):
    def update(self, id, updates, original):
        """
        Overriding to set the value of "new" attribute of rules to empty string if it's None.
        """

        for rule in updates.get("rules", {}):
            if rule["new"] is None:
                rule["new"] = ""

        return super().update(id, updates, original)

    def on_delete(self, doc):
        if self.backend.find_one("ingest_providers", req=None, rule_set=doc["_id"]):
            raise SuperdeskApiError.forbiddenError(_("Cannot delete Rule set as it's associated with channel(s)."))



import json

import apps.archive  # NOQA

# XXX: This import is needed in order to avoid ImportError when importing
# package_service, caused by circular dependencies.
# When that issue is resolved, the workaround should be removed.

import apps.packages.package_service as package
from flask import current_app as app
from superdesk import get_resource_service
from superdesk.services import BaseService
from eve.utils import ParsedRequest
from superdesk.notification import push_notification
from superdesk.utc import get_timezone_offset, utcnow
from eve.utils import config
from apps.archive.common import ITEM_MARK, ITEM_UNMARK


def init_parsed_request(elastic_query):
    parsed_request = ParsedRequest()
    parsed_request.args = {"source": json.dumps(elastic_query)}
    return parsed_request


def get_highlighted_items(highlights_id):
    """Get items marked for given highlight and passing date range query."""
    highlight = get_resource_service("highlights").find_one(req=None, _id=highlights_id)
    query = {
        "query": {
            "filtered": {
                "filter": {
                    "and": [
                        {
                            "range": {
                                "versioncreated": {
                                    "gte": highlight.get("auto_insert", "now/d"),
                                    "time_zone": get_timezone_offset(config.DEFAULT_TIMEZONE, utcnow()),
                                }
                            }
                        },
                        {"term": {"highlights": str(highlights_id)}},
                    ]
                }
            }
        },
        "sort": [
            {"versioncreated": "desc"},
        ],
        "size": 200,
    }
    request = ParsedRequest()
    request.args = {"source": json.dumps(query), "repo": "archive,published"}
    return list(get_resource_service("search").get(req=request, lookup=None))


def init_highlight_package(doc):
    """Add to package items marked for doc highlight."""
    main_group = doc.get("groups")[1]
    items = get_highlighted_items(doc.get("highlight"))
    used_items = []
    for item in items:
        if item["_id"] not in used_items:
            main_group["refs"].append(package.get_item_ref(item))
            used_items.append(item["_id"])


def init_default_content_profile(doc):
    if not doc.get("profile"):
        desk_id = doc.get("task", {}).get("desk")
        desk = get_resource_service("desks").find_one(req=None, _id=desk_id)
        doc["profile"] = desk.get("default_content_profile")


def on_create_package(sender, docs):
    """Call init_highlight_package for each package with highlight reference."""
    for doc in docs:
        if doc.get("highlight"):
            init_highlight_package(doc)
            init_default_content_profile(doc)


def get_highlight_name(highlights_id):
    """
    Given the id of a highlight it will return the name.
    :param hightlight_id:
    :return:
    """
    highlight = get_resource_service("highlights").find_one(req=None, _id=highlights_id)
    if highlight and "name" in highlight:
        return highlight.get("name", None)
    return None


class HighlightsService(BaseService):
    def on_delete(self, doc):
        service = get_resource_service("archive")
        highlights_id = str(doc["_id"])
        query = {"query": {"filtered": {"filter": {"term": {"highlights": highlights_id}}}}}
        req = init_parsed_request(query)
        proposedItems = service.get(req=req, lookup=None)
        for item in proposedItems:
            app.on_archive_item_updated(
                {"highlight_id": highlights_id, "highlight_name": get_highlight_name(highlights_id)}, item, ITEM_UNMARK
            )
            highlights = [h for h in item.get("highlights") if h != highlights_id]
            service.update(item["_id"], {"highlights": highlights}, item)


class MarkedForHighlightsService(BaseService):
    def create(self, docs, **kwargs):
        """Toggle highlight status for given highlight and item."""
        service = get_resource_service("archive")
        publishedService = get_resource_service("published")
        ids = []
        for doc in docs:
            item = service.find_one(req=None, _id=doc["marked_item"])
            if not item:
                ids.append(None)
                continue
            ids.append(item["_id"])
            highlights = item.get("highlights", [])
            if not highlights:
                highlights = []

            if doc["highlights"] not in highlights:
                highlights.append(doc["highlights"])
                highlight_on = True  # highlight toggled on
            else:
                highlights = [h for h in highlights if h != doc["highlights"]]
                highlight_on = False  # highlight toggled off

            updates = {"highlights": highlights, "_updated": item["_updated"], "_etag": item["_etag"]}
            service.update(item["_id"], updates, item)

            publishedItems = publishedService.find({"item_id": item["_id"]})
            for publishedItem in publishedItems:
                if publishedItem["_current_version"] == item["_current_version"] or not highlight_on:
                    updates = {
                        "highlights": highlights,
                        "_updated": publishedItem["_updated"],
                        "_etag": publishedItem["_etag"],
                    }
                    publishedService.update(publishedItem["_id"], updates, publishedItem)

            if highlight_on:
                app.on_archive_item_updated(
                    {"highlight_id": doc["highlights"], "highlight_name": get_highlight_name(doc["highlights"])},
                    item,
                    ITEM_MARK,
                )
            else:
                app.on_archive_item_updated(
                    {"highlight_id": doc["highlights"], "highlight_name": get_highlight_name(doc["highlights"])},
                    item,
                    ITEM_UNMARK,
                )

            push_notification(
                "item:highlights", marked=int(highlight_on), item_id=item["_id"], mark_id=str(doc["highlights"])
            )

        return ids


package.package_create_signal.connect(on_create_package)



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : Jérôme
# Creation: 2018-05-16 17:56

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "vocabularies"

    def forwards(self, mongodb_collection, mongodb_database):
        print(
            mongodb_collection.update_many(
                {"unique_field": {"$exists": False}, "schema.qcode": {"$exists": True}},
                {"$set": {"unique_field": "qcode"}},
            )
        )

    def backwards(self, mongodb_collection, mongodb_database):
        pass



import unittest

from apps.preferences import enhance_document_with_default_prefs


class DefaultUserPrefsTestCase(unittest.TestCase):
    def test_enhance(self):
        doc = {
            "user_preferences": {
                "workqueue:items": ["abc"],
                "editor:theme": {"label": "Editor"},
            }
        }
        enhance_document_with_default_prefs(doc)
        self.assertNotIn("label", doc["user_preferences"]["editor:theme"])
